#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2012, GoodData(R) Corporation. All rights reserved

import logging
lg = logging.getLogger('smoker')

import threading
import urllib2
import simplejson
import time
import datetime

from smoker.util.progressbar import ProgressBar, NonInteractiveError

class Client(object):
    """
    Object providing access to the group of hosts
    """
    hosts = []

    def __init__(self, hosts):
        """
        Create Host objects
        """
        assert isinstance(hosts, list), "Parameter hosts should be list"

        pool = []
        lg.info("Loading info for %d hosts" % len(hosts))
        for host in hosts:
            host = Host(host)

            t = threading.Thread(target=host.load_about)
            t.daemon = True
            t.start()
            pool.append(t)
            self.hosts.append(host)
        self.wait(pool)

        # Check if hosts are OK and remove
        # broken ones
        i = len(self.hosts) - 1
        while i >= 0:
            host = self.hosts[i]
            if not host.links:
                del(self.hosts[i])
                lg.info("Removing %s from list of hosts" % host.name)
            i -= 1

    def get_plugins(self, filters=None, filters_negative=False, exclude_plugins=None):
        """
        Return dictionary of each host and plugin
        where plugin name is the key

        Accept filter argument to filter plugins
        by parameters

        filter = {
            'key'  : 'Category',
            'value': 'system'
        }

        result = {
            'Airy' : {
                'plugin1' : {...},
                'plugin2' : {...},
            },
            'Host2': {...}
        }
        """
        result = {}
        lg.info("Getting plugins for %d hosts" % len(self.hosts))
        plugins = self.open(resource='plugins')
        result = self._format_plugins(plugins, filters=filters, filters_negative=filters_negative, exclude_plugins=exclude_plugins)
        return PluginsResult(**result)

    def force_run(self, plugins, progress=True):
        """
        Force plugins run
        """
        # Map dictionary hostname -> Host object for easier access to it
        host_map = {}
        for host in self.hosts:
            host_map[host.name] = host

        pool = []
        # For each host in plugins result, force run of it's plugins
        for hostname, host in plugins.iteritems():
            t = threading.Thread(name=hostname, target=host_map[hostname].force_run, args=(host['plugins'],))
            t.daemon = True
            t.start()
            pool.append(t)

        if progress:
            self.wait_progress(pool)
        else:
            self.wait(pool)

        result = {}
        for hostname, host in plugins.iteritems():
            result[hostname] = host_map[hostname].get_result()

        result = self._format_plugins(result)
        return PluginsResult(**result)

    def _format_plugins(self, plugins, filters=[], filters_negative=False, exclude_plugins=None):
        """
        Format plugins result by host and it's status, apply filtering

        Example:
            result = {
                'host1': {
                    'status' : 'OK',
                    'plugins': {
                        'plugin_name' : ...
                    }
                }
            }

        :param plugins: PluginsResult object (expected dict behavior)
        :param filters: list of filters for plugin.match_filters() function
        :param negative: True if filters should be negative
        :param exclude_plugins: list of plugin names to exclude
        """
        result = {}
        for name, host in plugins.iteritems():
            # No plugins for host, skip it
            try:
                host['plugins']['items']
            except:
                continue

            for plugin in host['plugins']['items']:
                plugin = plugin['plugin']

                # Filter plugins
                match = self._match_filters(plugin, filters, negative=filters_negative, exclude_plugins=exclude_plugins)

                # Passed filter, update result
                if match:
                    # Create result structure if it doesn't exists
                    try:
                        result[name]
                    except:
                        result[name] = {
                            'status' : None,
                            'plugins': {},
                        }
                    result[name]['plugins'][plugin['name']] = plugin

                    # Update host status
                    if plugin['lastResult']:
                        if plugin['lastResult']['status'] == 'ERROR':
                            result[name]['status'] = 'ERROR'
                        elif plugin['lastResult']['status'] == 'WARN' and result[name]['status'] != 'ERROR':
                            result[name]['status'] = 'WARN'
                        else:
                            if result[name]['status'] not in ['ERROR', 'WARN']:
                                result[name]['status'] = 'OK'
        return result

    def _match_filters(self, plugin, filters, negative=False, exclude_plugins=None):
        """
        Check if plugin passes supplied filters

        :param plugin: plugin object
        :param filters: list of filters, filters can be special (key,value tuple), by parameters (dict) or list of plugin names (list)
        :param negative: True if filters should be negative
        :param exclude_plugins: list of plugin names to exclude
        :rvalue: bool
        """
        # Check if plugin is excluded
        if exclude_plugins:
            if plugin['name'] in exclude_plugins:
                return False

        # return True or False depending on negative variable
        match_result = lambda match: not match if negative else match

        match = True
        for filter in filters:
            # Special filters
            if isinstance(filter, tuple):
                key, value = filter
                # Filter by plugin status
                if key == 'status' and isinstance(value, list):
                    try:
                        if plugin['lastResult']['status'] in value:
                            match = True
                            if not negative: continue
                            else: break
                        else:
                            match = False
                            break
                    except (KeyError, TypeError):
                        # Unknown state - no plugin['lastResult']['status'] or it's NoneType
                        if 'UNKNOWN' in value:
                            match = True
                            if not negative: continue
                            else: break
                        else:
                            match = False
                            break
            # Filter by parameters
            elif isinstance(filter, dict):
                try:
                    if plugin['parameters'][filter['key']] == filter['value']:
                        lg.debug("Plugin %s matched filter %s = %s" % (plugin['name'], filter['key'], filter['value']))
                        match = True
                        if not negative: continue
                        else: break
                    else:
                        lg.debug("Plugin %s doesn't match filter %s = %s" % (plugin['name'], filter['key'], filter['value']))
                        match = False
                        break
                except KeyError:
                    lg.debug("Plugin %s doesn't have filter parameter %s" % (plugin['name'], filter['key']))
                    match = False
                    break
            # Filter by list of plugins
            elif isinstance(filter, list):
                if plugin['name'] in filter:
                    lg.debug("Plugin %s matched requested plugins list" % plugin['name'])
                    match = True
                    continue
                else:
                    match = False
                    lg.debug("Plugin %s doesn't match requested plugins list" % plugin['name'])
                    break

        match = match_result(match)
        return match

    def open(self, uri=None, resource=None, data=None):
        """
        Open given uri in parallel and
        get JSON-parsed result
        """
        if not uri and not resource:
            raise Exception("Argument uri or resource have to be submitted")

        pool = []
        for host in self.hosts:
            t = threading.Thread(name=host.name, target=host.open, args=(uri, resource, data,))
            t.daemon = True
            t.start()
            pool.append(t)

        self.wait(pool)

        result = {}
        for host in self.hosts:
            result[host.name] = host.get_result()

        return result

    def wait(self, pool):
        """
        Wait until all threads in pool are done
        """
        done = False
        while not done:
            done = True
            for t in pool:
                if t.isAlive():
                    done = False
            time.sleep(0.5)

    def wait_progress(self, pool):
        """
        Wait until all threads in pool are done
        and show nice progress bar
        """
        try:
            with ProgressBar(len(pool)) as progress:
                progress.wait_pool(pool)
        except NonInteractiveError as e:
            # Fallback to non-progress wait
            lg.warn(e)
            self.wait(pool)

class Host(object):
    """
    Object representing single smokerd server
    """
    name = None
    url = None
    address = None
    links = {}

    _result  = None

    def __init__(self, address):
        """
        Initialize object
        """
        host = address.split(':')
        try:
            port = host[1]
        except IndexError:
            port = 8086

        self.name = address
        self.url = "http://%s:%s" % (host[0], port)
        self.address = address
        self.links = {}
        self._result = None

    def load_about(self):
        """
        Load informations from about page
        """
        about = self.open('/', timeout=5)
        if about:
            self.name = about['about']['host']

            for link in about['about']['links']:
                self.links[link['rel']] = link
        else:
            return False

        return about

    def open(self, uri=None, resource=None, data=None, timeout=20):
        """
        Open given uri and get JSON-parsed result
        """
        if not uri and not resource:
            raise Exception("Argument uri or resource have to be submitted")

        if resource:
            try:
                uri = self.links[resource]['href']
            except KeyError:
                lg.error("Can't find resource %s" % resource)
                return False

        url = '%s%s' % (self.url, uri)
        lg.info("Host %s: requesting url %s" % (self.name, url))
        try:
            fh = urllib2.urlopen(url, timeout=timeout, data=data)
        except Exception as e:
            lg.error("Host %s: can't open resource %s: %s" % (self.name, url, e))
            return False

        try:
            json = simplejson.load(fh)
        except Exception as e:
            lg.error("Host %s: can't load response as JSON: %s" % (self.name, e))
            return False

        self._result = json
        return json

    def force_run(self, plugins):
        """
        Force plugin run
        Poll process until result

        plugins = {
            'pluginName' : {...},
            'pluginName2': {...},
        }
        """
        plugins_list = plugins.keys()
        data = simplejson.dumps({
            'process' : {
                'plugins' : plugins_list,
            }
        })

        lg.info("Forcing run of %d plugins on host %s" % (len(plugins_list), self.name))
        process = self.open(resource='processes', data=data)

        if process:
            poll = process['asyncTask']['link']['poll']
            return self.poll(uri=poll)
        else:
            return False

    def poll(self, uri, sleep=1):
        result = None
        retries = 3

        # Poll process for result
        while not result:
            res = self.open(uri=uri)

            if not res:
                if retries == 0:
                    lg.error(
                        "Polling on %s failed after 3 retries. This may "
                        "happen when a plugin died while waiting for the "
                        "result. Please retry and check log on the hosts if "
                        "it happens again." % uri)
                    return False
                else:
                    retries -= 1
                    continue

            if res.has_key('asyncTask'):
                uri = res['asyncTask']['link']['poll']
                time.sleep(1)
            else:
                result = res
        return result


    def get_result(self):
        """
        Return result and clear it
        """
        result = self._result
        self._result = None
        return result

class PluginsResult(dict):
    """
    Object for Client results
    """

    def __init__(self, *args, **kwargs):
        super(PluginsResult, self).__init__(*args, **kwargs)

        # Convert self structure to use PluginResult and HostResult instead of simple dict
        for name in self:
            self[name] = HostResult(self[name])
            for key in self[name]['plugins']:
                self[name]['plugins'][key] = PluginResult(self[name]['plugins'][key])

    def count_hosts(self):
        """
        Print number of hosts in result
        """
        return len(self)

    def get_host_plugins(self, host=None):
        """
        Return list of tuples with objects like:
        [(HostResult, PluginResult)]
        """
        result = []

        if not host:
            # Return all hosts
            for name in sorted(self):
                host = self[name]
                host['name'] = name
                for key in sorted(host['plugins']):
                    plugin = host['plugins'][key]
                    result.append((host, plugin))
        else:
            # Return only one host
            host = self[host]
            for key in sorted(host['plugins']):
                plugin = host['plugins'][key]
                result.append((host, plugin))
        return result

class PluginResult(dict):
    """
    Object for plugin result
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize dict and fix plugin structure
        """
        super(PluginResult, self).__init__(*args, **kwargs)
        self._fix_plugin()

    def _fix_plugin(self):
        """
        Fix plugin structure
        """
        # If plugin doesn't have lastResult, set it as unknown
        if not self['lastResult']:
            self['lastResult'] = {
                'status'   : 'UNKNOWN',
                'messages' : None,
                'componentResults' : None,
                'lastRun'  : None,
                }
        else:
            # convert lastRun to datetime object
            self['lastResult']['lastRun'] = datetime.datetime.strptime(self['lastResult']['lastRun'].partition('.')[0], "%Y-%m-%dT%H:%M:%S")

        if self['nextRun']:
            self['nextRun'] = datetime.datetime.strptime(self['nextRun'].partition('.')[0], "%Y-%m-%dT%H:%M:%S")

        return self

class HostResult(dict):
    """
    Object for Host result
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize dict and fix host structure
        """
        super(HostResult, self).__init__(*args, **kwargs)
        self._fix_host()

    def _fix_host(self):
        """
        Fix host structure
        """
        if not self['status']:
            self['status'] = 'UNKNOWN'

        return self



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2013, GoodData(R) Corporation. All rights reserved

"""
Module with various console utils
"""

import sys
import fcntl, termios, struct

def get_terminal_size():
    """
    Get terminal width and height
    Return (width, height) tuple
    """
    h, w, hp, wp = struct.unpack('HHHH',
        fcntl.ioctl(0, termios.TIOCGWINSZ,
            struct.pack('HHHH', 0, 0, 0, 0)))
    return w, h

def is_interactive_shell():
    """
    Try to guess if current shell is interactive
    Return bool
    """
    return sys.__stdout__.isatty()


# -*- coding: utf-8 -*-
# Copyright © 2007-2013, All rights reserved. GoodData® Corporation

__author__ = "miroslav.hedl@gooddata.com"
__maintainer__ = __author__


'''
Primary goal of this module is function `plugins_to_xml` that solves issue
PCI-1385.

This module converts plugins dictionary from `Smoker.py` and produces generic
list of named tuple based on configuration template (described inside).

Also implements simple but flexible
:py:class:`HtmlBuilder('node_name').sub_node(attr1='val1',
attr2='$GenericValue').innerText("Inside text...")`

Jenkins doesn't take structure of xUnit/jUnit XML file into account.
So sorting or structuring testsuites/testcases is futile effort.

Important is to properly set 'classname' and 'name' attributes for testcase
elements.
'''

from cgi import escape
import collections
import yaml

import default_config
import rows
from xml_builder import XmlBuilder


def plugins_to_xml(dict_data,
                   yaml_filename=None,
                   yaml_data=default_config.YAML_CONFIG,
                   dict_templates=['All'],
                   additional_fields='AdditionalFields',
                   ts_attr='HtmlTestSuiteAttr',
                   tc_attr='HtmlTestCaseAttr',
                   tc_elem='HtmlTestCaseElem'):
    """
    Provided data (from plugins dictionary) and walking template, get all valid
    items and convert it to jUnit xml representation.

    Function have sane defaults (depends on calee opinion):
    :param dict dict_data: datastructure taken as result from running smoke
                           tests (Smoker's output)
    :param str yaml_data: yaml string that will be taken as configuration; does
                          have precedence before `yaml_filename`
    :param str yaml_filename: if yaml_data is None, tries to read config from
                              file specified as path (relative from cwd)
    :param list dict_templates: TODO: probably remove this parameter
    :param dict additional_fields: get Yaml name for additional fields for Row
                                   namedtuple
    :param dict ts_attr: get Yaml name for configured testsuite xml attributes
    :param dict tc_attr: get Yaml name for configured testcase xml attributes
    :param dict tc_elem: get Yaml name for configured testcase xml subelements

    :rval: string
    :return: returns xml structure (testsuites corresponds to nodes, testcases
             to plugin)
    """
    def apply(inst, custom_dict=None, **kwargs):
        """
        Dynamically applies value of value as new value.

        >>> inst
        Row(node='stg-c3', plugin='alog', status='UNKNOWN')
        >>> inst.ClassName
        'stg-c3.alog'
        >>> custom_dict
        { 'name': 'node', 'classname': 'ClassName'}
        >>> apply(inst, custom_dict=custom_dict)
        { 'name': 'sgt-c3', 'classname': 'stg-c3.alog'}
        """
        applied_args = {}
        if custom_dict:
            for k, v in custom_dict.iteritems():
                applied_args[k] = getattr(inst, v)
        for k, v in kwargs.iteritems():
            applied_args[k] = getattr(inst, v)
        return applied_args

    if yaml_filename:
        with open(yaml_filename) as f:
            C = yaml.safe_load(f)
    else:
        C = yaml.safe_load(yaml_data)

    results = {}
    for template in dict_templates:
        results[template] = rows.create(data=dict_data,
                                        template=C[template],
                                        additional_fields=C[additional_fields])

    ts_data = {}
    for res in results.iterkeys():
        ts_data[res] = collections.defaultdict(list)
        ts_res = ts_data[res]
        for row in results[res]:
            ts_res[row.Node].append(row)

    junit_xml = XmlBuilder()
    for template_name, ts in sorted(ts_data.iteritems()):
        with junit_xml.testsuites as html_tss:
            html_tss(name=template_name)
            for node, tcs in sorted(ts.iteritems()):
                with html_tss.testsuite as html_ts:
                    first = tcs[0] if tcs else None
                    if first:
                        html_ts(custom_dict=apply(first,
                                                  custom_dict=C[ts_attr]))
                    for tc in sorted(tcs):
                        html_tc = html_ts.testcase(
                            custom_dict=apply(tc, custom_dict=C[tc_attr]))

                        # handle plugins without components
                        if tc.CaseStatus:
                            distinguisher = tc.CaseStatus
                        else:
                            distinguisher = tc.PluginStatus
                        if not tc.CaseName:
                            html_tc.name = tc.Plugin

                        if distinguisher == 'ERROR':
                            if tc.MsgError:
                                html_tc.error(message=escape(
                                    list_to_string(tc.MsgError), quote=1))
                            else:
                                html_tc.error()
                        elif distinguisher == 'WARN':
                            # add the child element
                            # can't call directly because of the dash
                            out = html_tc.__getattr__('system-out')
                            if tc.MsgWarn:
                                # poppulate it with content, fi applicable
                                out.__setattr__('message', escape(
                                    list_to_string(tc.MsgWarn), quote=1))
    return junit_xml.dump()


def list_to_string(message):
    if isinstance(message, list):
        return '\n'.join(message)
    else:
        return message

# def test_onepass():
#     fixture_filename = 'tests/input_data_fixture'
#
#     with open('%s.yaml' % (fixture_filename,)) as f_in:
#         xml_txt = plugins_to_xml(yaml.safe_load(f_in))
#
#     with open('%s.testresult' % (fixture_filename,), 'w') as f_out:
#         f_out.write(xml_txt)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2012, GoodData(R) Corporation. All rights reserved

"""
Parser varnishparser is used to parse
output from varnishadm 'debug.health'

If output can't be parsed, it will raise exception
and default result output will be used
"""

import re

from smoker.server.parser import BaseParser


class Parser(BaseParser):
    def parse(self):
        """
        If we got stdout, parse it and return result.
        Else return Exception that is handled by smokerd.
        """
        backends = re.findall("Backend\ (.*)\ is\ (.*)\n.*\nAverage responsetime of good probes:\ (.*)", self.stdout)

        if not backends:
            raise Exception("No backends found or output can't be parsed!")

        for backend in backends:
            name    = backend[0]
            state   = backend[1]
            restime = float(backend[2])

            args = {
                'info' : [],
                'error': [],
                'warn' : [],
            }

            # Check backend state
            if state == 'Healthy':
                # Check backend response time
                if restime > 0.5:
                    state = 'WARN'
                    args['warn'].append('Response time: %s' % restime)
                else:
                    state = 'OK'
                    args['info'].append('Response time: %s' % restime)
            else:
                state = 'ERROR'
                args['error'].append('State: %s' % backend[1])

            self.result.add_component(name, state, **args)

        self.result.set_status()

        return self.result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2012, GoodData(R) Corporation. All rights reserved

"""
Client tool for GDC Smoker daemon

See pydoc for command line tool smoker.py
"""

import argparse
import datetime
import glob
import logging
import os
import simplejson
import sys
import yaml

from smoker.client import Client
import smoker.logger
from smoker.client.out_junit import plugins_to_xml
from smoker.util.tap import TapTest, Tap


smoker.logger.init(syslog=False)
lg = logging.getLogger('smokercli')


COLORS = {
    'default' : '\033[0;0m',
    'magenta' : '\033[1;35m',
    'yellow'  : '\033[1;33m',
    'gold' : '\033[0;33m',
    'red'  : '\033[1;31m',
    'blue' : '\033[1;34m',
    'green': '\033[1;32m',
    'cyan' : '\033[1;36m',
    'gray' : '\033[0;37m',
}


CONFIG_FILE = '/etc/smokercli.yaml'


def _get_plugins():
    """
    Get list of available host discovery plugin module names
    """

    plugins = []
    conf_file = os.path.expanduser('~/.smokercli.yaml')

    if not os.path.exists(conf_file):
        conf_file = CONFIG_FILE

    if not os.path.exists(conf_file):
        return plugins

    with open(conf_file) as f:
        config = yaml.safe_load(f)

    if config and 'plugin_paths' in config:
        paths = config['plugin_paths']
    else:
        raise Exception('Invalid config file')


    for path in paths:
        try:
            module = __import__(path)
        except ImportError:
            raise Exception('Invalid config file')

        toplevel = os.path.dirname(module.__file__)
        submodule = '/'.join(path.split('.')[1:])
        plugin_dir = os.path.join(toplevel, submodule, '*.py')
        modules = [os.path.basename(name)[:-3] for name in
                   glob.glob(plugin_dir)]
        modules.remove('__init__')
        plugins += ['%s.%s' % (path, name) for name in modules]

    return plugins


def _get_plugin_arguments(name):
    """
    Get list of host discovery plugin specific cmdline arguments

    :param name: plugin module name
    """
    try:
        plugin = __import__(name, globals(), locals(), ['HostDiscoveryPlugin'])
    except ImportError as e:
        lg.error("Can't load module %s: %s" % (name, e))
        raise
    return plugin.HostDiscoveryPlugin.arguments


def _add_plugin_arguments(parser):
    """
    Add host discovery plugin specific options to the cmdline argument parser

    :param parser: argparse.ArgumentParser instance
    """

    plugins = _get_plugins()
    if not plugins:
        return
    argument_group = parser.add_argument_group('Plugin arguments')

    for plugin in plugins:
        args = _get_plugin_arguments(plugin)
        for argument in args:
            argument_group.add_argument(*argument.args, **argument.kwargs)


def _run_discovery_plugin(name, args):
    """
    Run the host discovery plugin
    :param name: plugin module name
    :param args: attribute namespace
    :return: discovered hosts list
    """
    try:
        this_plugin = __import__(name, globals(), locals(),
                                 ['HostDiscoveryPlugin'])
    except ImportError as e:
        lg.error("Can't load module %s: %s" % (name, e))
        raise

    plugin=this_plugin.HostDiscoveryPlugin()
    return plugin.get_hosts(args)


def _host_discovery(args):
    """
    Run all the discovery plugins

    :param args: attribute namespace
    :return: discovered hosts list
    """
    discovered = []

    for plugin in _get_plugins():
        hosts = _run_discovery_plugin(plugin, args)
        if hosts:
            discovered += hosts

    return discovered


def main():
    """
    Main entrance
    """
    parser = argparse.ArgumentParser(
        description='Smoker client tool', add_help=False)

    # Action arguments
    group_action = parser.add_argument_group('Action switchers')
    group_action.add_argument(
        '-f', '--force', dest='force', action='store_true',
        help="Force plugins run (otherwise just print last results)")
    group_action.add_argument(
        '-l', '--list', dest='list', action='store_true', help="List plugins")
    group_action.add_argument(
        '-h', '--help', dest='help', action='store_true',
        help="Show this help and exit")

    # Host arguments
    group_main = parser.add_argument_group('Target host switchers')
    group_main.add_argument(
        '-s', '--hosts', dest='hosts', nargs='+',
        help="Hosts with running smokerd (default localhost)")

    # Filtering options
    # List of plugins
    group_filters = parser.add_argument_group('Filters')
    group_filters.add_argument(
        '-p', '--plugins', dest='plugins', nargs='+',
        help="Filter plugins by name")
    group_filters.add_argument(
        '--exclude-plugins', dest='exclude_plugins', nargs='+',
        help="Exclude plugins by names")

    # Other filters
    group_filters.add_argument(
        '--filter', dest='filter', default=[], nargs='+',
        help=("Filter plugins by it's parameters, "
              "eg. --filter 'Category connectors'"))
    group_filters.add_argument(
        '--exclude', action='store_true',
        help=("Negative filters (exclude) - currently works only "
              "for one filter"))
    group_filters.add_argument(
        '--category', help="Filter plugins by Category parameter")
    group_filters.add_argument(
        '--component',
        help="Filter plugins by Component parameter (eg. server)")
    group_filters.add_argument(
        '--health', action='store_true',
        help="Filter plugins by Type healthCheck")
    group_filters.add_argument(
        '--smoke', action='store_true',
        help="Filter plugins by Type smokeTest")

    # Plugin state filters
    group_filters.add_argument(
        '--filter-status', dest='filter_status', default=[], nargs='+',
        help="Filter plugins by it's state, eg. --filter-state ERROR WARN")
    group_filters.add_argument(
        '--nook', dest='state_nook', action='store_true',
        help=("Only non-OK plugins (ERROR, WARN, UNKNOWN). "
              "Can't be used together with --filter-state"))
    group_filters.add_argument(
        '--error', dest='state_error', action='store_true',
        help="Only ERROR. Can't be used together with --filter-state")
    group_filters.add_argument(
        '--warn', '--warning', dest='state_warn', action='store_true',
        help="Only WARN. Can't be used together with --filter-state")
    group_filters.add_argument(
        '--unknown', dest='state_unknown', action='store_true',
        help="Only UNKNOWN. Can't be used together with --filter-state")
    group_filters.add_argument(
        '--ok', dest='state_ok', action='store_true',
        help="Only OK. Can't be used together with --filter-state")

    # Output switchers
    group_output = parser.add_argument_group('Output switchers')
    group_output.add_argument(
        '-o', '--pretty', dest='pretty', default='normal',
        help=("Output format: minimal / normal / long / full / raw / json / "
              "tap / xml"))
    group_output.add_argument(
        '--no-colors', dest='no_colors', action='store_true',
        help="Don't use colors in output")
    group_output.add_argument(
        '--no-progress', dest='no_progress', action='store_true',
        help="Don't show progress bar")
    group_output.add_argument(
        '-v', '--verbose', dest='verbose', action='store_true',
        help="Be verbose")
    group_output.add_argument(
        '-d', '--debug', dest='debug', action='store_true',
        help="Debug output")
    group_output.add_argument(
        '--junit-config-file', dest='junit_config_file',
        help="Name of configuration file for junit xml formatter")

    _add_plugin_arguments(parser)
    args = parser.parse_args()

    # Set log level and set args.no_progress option
    if args.verbose:
        lg.setLevel(logging.INFO)
        args.no_progress = True

    if args.debug:
        lg.setLevel(logging.DEBUG)
        args.no_progress = True

    if args.help:
        parser.print_help()
        sys.exit(0)

    if args.pretty == 'minimal':
        # Minimal output (only host and errored plugins)
        if not args.no_colors:
            format_host   = {
                'OK'    : '%(gold)s{name:<34}%(default)s [%(green)s{status}%(default)s]'  % COLORS,
                'WARN'  : '%(gold)s{name:<34}%(default)s [%(yellow)s{status}%(default)s]'  % COLORS,
                'ERROR' : '%(gold)s{name:<34}%(default)s [%(red)s{status}%(default)s]'  % COLORS,
                'UNKNOWN' : '%(gold)s{name:<34}%(default)s [%(magenta)s{status}%(default)s]'  % COLORS,
            }
            format_plugin = {
                'OK'    : '',
                'WARN'  : '- {name:<32} [%(yellow)s{lastResult[status]}%(default)s]   (%(gray)s{lastResult[lastRun]}%(default)s)'  % COLORS,
                'ERROR' : '- {name:<32} [%(red)s{lastResult[status]}%(default)s] (%(gray)s{lastResult[lastRun]}%(default)s)'  % COLORS,
                'UNKNOWN' : '- {name:<32} [%(magenta)s{lastResult[status]}%(default)s]'  % COLORS
            }
            format_plugin_component = {
                'OK'    : '',
                'WARN'  : ' -- {name:<30} [%(yellow)s{status}%(default)s]'  % COLORS,
                'ERROR' : ' -- {name:<30} [%(red)s{status}%(default)s]'  % COLORS,
                'UNKNOWN' : ' -- {name:<30} [%(magenta)s{status}%(default)s]'  % COLORS,
            }
        else:
            format_host   = '{name:<34} [{status}]'
            format_plugin = {
                'OK'    : '',
                'WARN'  : '- {name:<32} [{lastResult[status]}] ({lastResult[lastRun]})',
                'ERROR' : '- {name:<32} [{lastResult[status]}] ({lastResult[lastRun]})',
                'UNKNOWN' : '- {name:<32} [{lastResult[status]}]'
            }
            format_plugin_component = {
                'OK'    : '',
                'WARN'  : ' -- {name:<30} [{status}]',
                'ERROR' : ' -- {name:<30} [{status}]',
                'UNKNOWN' : ' -- {name:<30} [{status}]',
            }

        format_plugin_msg = ''
        format_plugin_component_msg = ''
        format_plugin_run = ''
        format_plugin_param = ''
    elif args.pretty == 'normal':
        # Normal output (host and it's plugins without component results)
        if not args.no_colors:
            format_host   = {
                'OK'    : '%(gold)s{name:<34}%(default)s [%(green)s{status}%(default)s]'  % COLORS,
                'WARN'  : '%(gold)s{name:<34}%(default)s [%(yellow)s{status}%(default)s]'  % COLORS,
                'ERROR' : '%(gold)s{name:<34}%(default)s [%(red)s{status}%(default)s]'  % COLORS,
                'UNKNOWN' : '%(gold)s{name:<34}%(default)s [%(magenta)s{status}%(default)s]'  % COLORS,
            }
            format_plugin = {
                'OK'   : '- {name:<32} [%(green)s{lastResult[status]}%(default)s]    (%(gray)s{lastResult[lastRun]}%(default)s)'  % COLORS,
                'WARN' : '- {name:<32} [%(yellow)s{lastResult[status]}%(default)s]   (%(gray)s{lastResult[lastRun]}%(default)s)'  % COLORS,
                'ERROR': '- {name:<32} [%(red)s{lastResult[status]}%(default)s] (%(gray)s{lastResult[lastRun]}%(default)s)'  % COLORS,
                'UNKNOWN': '- {name:<32} [%(magenta)s{lastResult[status]}%(default)s]'  % COLORS
            }
            format_plugin_msg = {
                'info'  : '',
                'warn'  : '  [%(yellow)s{level}%(default)s] {msg}'  % COLORS,
                'error' : '  [%(red)s{level}%(default)s] {msg}'  % COLORS,
            }
            format_plugin_component = {
                'OK'    : '',
                'WARN'  : ' -- {name:<30} [%(yellow)s{status}%(default)s]'  % COLORS,
                'ERROR' : ' -- {name:<30} [%(red)s{status}%(default)s]'  % COLORS,
                'UNKNOWN' : ' -- {name:<30} [%(magenta)s{status}%(default)s]'  % COLORS,
            }
            format_plugin_component_msg = {
                'info'  : '',
                'warn'  : '    [%(yellow)s{level}%(default)s] {msg}'  % COLORS,
                'error' : '    [%(red)s{level}%(default)s] {msg}'  % COLORS,
            }
        else:
            format_host   = '{name:<34} [{status}]'
            format_plugin = '- {name:<32} [{lastResult[status]}] ({lastResult[lastRun]})'
            format_plugin_msg = {
                'info'  : '',
                'warn'  : '  [{level}] {msg}',
                'error' : '  [{level}] {msg}',
            }
            format_plugin_component = {
                'OK'    : '',
                'WARN'  : ' -- {name:<30} [{status}]',
                'ERROR' : ' -- {name:<30} [{status}]',
                'UNKNOWN' : ' -- {name:<30} [{status}]',
            }
            format_plugin_component_msg = {
                'info'  : '',
                'warn'  : '    [{level}] {msg}',
                'error' : '    [{level}] {msg}',
            }

        format_plugin_run = ''
        format_plugin_param = ''
    elif args.pretty == 'long':
        if not args.no_colors:
            format_host   = {
                'OK'    : '%(gold)s{name:<34}%(default)s [%(green)s{status}%(default)s]'  % COLORS,
                'WARN'  : '%(gold)s{name:<34}%(default)s [%(yellow)s{status}%(default)s]'  % COLORS,
                'ERROR' : '%(gold)s{name:<34}%(default)s [%(red)s{status}%(default)s]'  % COLORS,
                'UNKNOWN' : '%(gold)s{name:<34}%(default)s [%(magenta)s{status}%(default)s]'  % COLORS,
            }
            format_plugin = {
                'OK'   : '- {name:<32} [%(green)s{lastResult[status]}%(default)s]    (%(gray)s{lastResult[lastRun]}%(default)s)'  % COLORS,
                'WARN' : '- {name:<32} [%(yellow)s{lastResult[status]}%(default)s]   (%(gray)s{lastResult[lastRun]}%(default)s)'  % COLORS,
                'ERROR': '- {name:<32} [%(red)s{lastResult[status]}%(default)s]    (%(gray)s{lastResult[lastRun]}%(default)s)'  % COLORS,
                'UNKNOWN': '- {name:<32} [%(magenta)s{lastResult[status]}%(default)s]'  % COLORS
            }
            format_plugin_msg = {
                'info'  : '  [%(blue)s{level}%(default)s] {msg}'  % COLORS,
                'warn'  : '  [%(yellow)s{level}%(default)s] {msg}'  % COLORS,
                'error' : '  [%(red)s{level}%(default)s] {msg}'  % COLORS,
            }
            format_plugin_component = {
                'OK'    : ' -- {name:<30} [%(green)s{status}%(default)s]'  % COLORS,
                'WARN'  : ' -- {name:<30} [%(yellow)s{status}%(default)s]'  % COLORS,
                'ERROR' : ' -- {name:<30} [%(red)s{status}%(default)s]'  % COLORS,
                'UNKNOWN' : ' -- {name:<30} [%(magenta)s{status}%(default)s]'  % COLORS,
            }
            format_plugin_component_msg = {
                'info'  : '    [%(blue)s{level}%(default)s] {msg}'  % COLORS,
                'warn'  : '    [%(yellow)s{level}%(default)s] {msg}'  % COLORS,
                'error' : '    [%(red)s{level}%(default)s] {msg}'  % COLORS,
            }
        else:
            format_host   = '{name:<34} [{status}]'
            format_plugin = '- {name:<32} [{lastResult[status]}] ({lastResult[lastRun]})'
            format_plugin_msg = '  [{level}] {msg}'
            format_plugin_component = ' -- {name:<30} [{status}]'
            format_plugin_component_msg = '    [{level}] {msg}'

        format_plugin_run = ''
        format_plugin_param = ''
    elif args.pretty == 'full':
        if not args.no_colors:
            format_host   = {
                'OK'    : '%(gold)s{name:<34}%(default)s [%(green)s{status}%(default)s]'  % COLORS,
                'WARN'  : '%(gold)s{name:<34}%(default)s [%(yellow)s{status}%(default)s]'  % COLORS,
                'ERROR' : '%(gold)s{name:<34}%(default)s [%(red)s{status}%(default)s]'  % COLORS,
                'UNKNOWN' : '%(gold)s{name:<34}%(default)s [%(magenta)s{status}%(default)s]'  % COLORS,
            }
            format_plugin = {
                'OK'   : '- {name:<32} [%(green)s{lastResult[status]}%(default)s]    (%(gray)s{lastResult[lastRun]}%(default)s)'  % COLORS,
                'WARN' : '- {name:<32} [%(yellow)s{lastResult[status]}%(default)s]   (%(gray)s{lastResult[lastRun]}%(default)s)'  % COLORS,
                'ERROR': '- {name:<32} [%(red)s{lastResult[status]}%(default)s]    (%(gray)s{lastResult[lastRun]}%(default)s)'  % COLORS,
                'UNKNOWN': '- {name:<32} [%(magenta)s{lastResult[status]}%(default)s]'  % COLORS
            }
            format_plugin_msg = {
                'info'  : '  [%(blue)s{level}%(default)s] {msg}'  % COLORS,
                'warn'  : '  [%(yellow)s{level}%(default)s] {msg}'  % COLORS,
                'error' : '  [%(red)s{level}%(default)s] {msg}'  % COLORS,
            }
            format_plugin_component = {
                'OK'    : ' -- {name:<30} [%(green)s{status}%(default)s]'  % COLORS,
                'WARN'  : ' -- {name:<30} [%(yellow)s{status}%(default)s]'  % COLORS,
                'ERROR' : ' -- {name:<30} [%(red)s{status}%(default)s]'  % COLORS,
                'UNKNOWN' : ' -- {name:<30} [%(magenta)s{status}%(default)s]'  % COLORS,
            }
            format_plugin_component_msg = {
                'info'  : '    [%(blue)s{level}%(default)s] {msg}'  % COLORS,
                'warn'  : '    [%(yellow)s{level}%(default)s] {msg}'  % COLORS,
                'error' : '    [%(red)s{level}%(default)s] {msg}'  % COLORS,
            }
        else:
            format_host   = '{name:<34} [{status}]'
            format_plugin = '- {name:<32} [{lastResult[status]}] ({lastResult[lastRun]})'
            format_plugin_msg = '  [{level}] {msg}'
            format_plugin_component = ' -- {name:<30} [{status}]'
            format_plugin_component_msg = '    [{level}] {msg}'

        format_plugin_run = '  Last run: {lastResult[lastRun]!s}\n  Next run: {nextRun!s}'
        format_plugin_param = '  {key:<10} {value}'
    elif args.pretty in ['raw', 'json', 'tap', 'xml']:
        # Raw and special outputs doesn't need formatting
        pass
    else:
        lg.error("Invalid pretty output %s" % args.pretty)
        sys.exit(1)

    # Setup custom filters
    if args.category:
        args.filter.append('Category %s' % args.category)

    if args.component:
        args.filter.append('Component %s' % args.component)

    # Health checks and smoke tests together? Don't add filter
    if args.health or args.smoke:
        if args.health:
            args.filter.append('Type healthCheck')

        if args.smoke:
            args.filter.append('Type smokeTest')

    # Setup plugins filter
    filters = []
    if args.filter:
        for f in args.filter:
            filter = {}
            try:
                filter = {
                    'key'  : f.split(' ')[0],
                    'value': f.split(' ')[1]
                }
            except:
                lg.error("Invalid filter parameter format!")
                sys.exit(1)

            filters.append(filter)

    if filters and args.plugins:
        lg.warn("Plugin filter and plugin list used together. "
                "Only plugins matching both will be displayed.")

    if args.plugins:
        filters.append(args.plugins)

    # Setup state filters
    states = []
    if args.filter_status:
        states = args.filter_status
    elif args.state_nook:
        states = ['ERROR', 'WARN', 'UNKNOWN']
    else:
        if args.state_ok:
            states.append('OK')
        if args.state_error:
            states.append('ERROR')
        if args.state_warn:
            states.append('WARN')
        if args.state_unknown:
            states.append('UNKNOWN')

    # State list cannot be empty
    if not states:
        states = ['OK', 'ERROR', 'WARN', 'UNKNOWN']
    # Add status filter
    filters.append(('status', states))

    hosts = ['localhost']
    discovered_hosts = _host_discovery(args)
    if args.hosts:
        hosts = args.hosts
        if discovered_hosts:
            hosts += discovered_hosts
    elif discovered_hosts:
        hosts = discovered_hosts

    # Initialize Client
    client = Client(hosts)
    plugins = client.get_plugins(filters, filters_negative=args.exclude,
                                 exclude_plugins=args.exclude_plugins)

    # No plugins found
    if not plugins:
        lg.error("No plugins found")
        sys.exit(1)

    # List plugins only (parameter --force is ignored)
    if args.list:
        plugins_tuple = plugins.get_host_plugins()

        if plugins.count_hosts() > 1:
            # We have more hosts than 1, print structure like:
            # server1/httpd
            # server2/crond
            for host, plugin in plugins_tuple:
                print "%s/%s" % (host['name'], plugin['name'])
        else:
            # Print only plugin-by-line structure without hostname
            for host, plugin in plugins_tuple:
                print "%s" % plugin['name']
        sys.exit(0)

    # Force plugins run
    # set progress=False if --no-progress parameter is set
    if args.force:
        plugins = client.force_run(
            plugins, progress=not args.no_progress)

    # Print raw output
    if args.pretty == 'raw':
        from pprint import pprint
        pprint(plugins)
        sys.exit(0)
    elif args.pretty == 'json':
        # We need custom encoder to encode datetime objects
        class JSONEncoder(simplejson.JSONEncoder):
            """
            JSON encoder that converts datetime.datetime object to isoformat
            string
            """
            def default(self, obj):
                if isinstance(obj, datetime.datetime):
                    return obj.isoformat()
                return simplejson.JSONEncoder.default(self, obj)

        print simplejson.dumps(plugins, cls=JSONEncoder)
        sys.exit(0)
    elif args.pretty == 'tap':
        dump = dump_tap(plugins)
        # Convert mixed string into ascii to workaround UnicodeEncodeError
        # shouldn't be needed in Python 3
        print dump.encode('ascii', 'ignore')
        sys.exit(0)
    elif args.pretty == 'xml':
        dump = plugins_to_xml(plugins, args.junit_config_file)
        print dump
        sys.exit(0)

    # Print result
    output = []
    hosts_printed = []
    for host, plugin in plugins.get_host_plugins():
        # Print host if not already printed
        if host['name'] not in hosts_printed:
            # Add empty line if any previous host
            if hosts_printed: output.append(" ")
            if isinstance(format_host, dict):
                output.append(format_host[host['status']].format(**host))
            else:
                output.append(format_host.format(**host))
            hosts_printed.append(host['name'])

        # Print plugin
        if not isinstance(plugin, dict):
            # Not a plugin
            continue
        if isinstance(format_plugin, basestring):
            output.append(format_plugin.format(**plugin))
        else:
            output.append(format_plugin[plugin['lastResult']['status']].format(**plugin))

        # Print last and next plugin run
        output.append(format_plugin_run.format(**plugin))

        # Print last and next plugin run
        for key, value in plugin['parameters'].iteritems():
            output.append(format_plugin_param.format(key=key, value=value))

        # Print plugin messages
        if plugin['lastResult']['messages']:
            # For each message level
            for level, message in plugin['lastResult']['messages'].iteritems():
                # For each message
                for msg in message:
                    if isinstance(format_plugin_msg, basestring):
                        output.append(format_plugin_msg.format(level=level, msg=msg.encode('utf8')))
                    else:
                        output.append(format_plugin_msg[level].format(level=level, msg=msg.encode('utf8')))

        # Print component result
        if plugin['lastResult']['componentResults']:
            for component in plugin['lastResult']['componentResults']:
                component = component['componentResult']
                if isinstance(format_plugin_component, basestring):
                    output.append(format_plugin_component.format(**component))
                else:
                    output.append(format_plugin_component[component['status']].format(**component))

                # Print component messages
                if component['messages']:
                    # For each message level
                    for level, message in component['messages'].iteritems():
                        # For each message
                        for msg in message:
                            if isinstance(format_plugin_component_msg, basestring):
                                output.append(format_plugin_component_msg.format(level=level, msg=msg.encode('utf8')))
                            else:
                                output.append(format_plugin_component_msg[level].format(level=level, msg=msg.encode('utf8')))

    for line in output:
        if line:
            print line

def dump_tap(plugins):
    """
    Dump plugins result to TAP
    Take OK and also WARN statuses as ok
    Print only error and warn results
    """
    tap = Tap()

    for name in sorted(plugins):
        host = plugins[name]
        if host['status'] in ['OK', 'WARN']:
            host_ok = True
        else:
            host_ok = False

        tap_host = TapTest(name, host_ok)
        tap.add_test(tap_host)

        # For each host's plugin
        for key in sorted(host['plugins']):
            plugin = host['plugins'][key]
            if not plugin['lastResult']:
                plugin_ok = False
            else:
                if plugin['lastResult']['status'] in ['OK', 'WARN']:
                    plugin_ok = True
                else:
                    plugin_ok = False

            # Add messages to test result
            messages = []
            if plugin['lastResult']:
                if plugin['lastResult']['messages']:
                    messages = plugin['lastResult']['messages']

            tap_plugin = TapTest(plugin['name'], plugin_ok, messages)
            tap_host.add_subtest(tap_plugin)

            if (plugin['lastResult'] and
                    plugin['lastResult']['componentResults']):
                # For each component result
                for component in plugin['lastResult']['componentResults']:
                    component = component['componentResult']
                    if component['status'] in ['OK', 'WARN']:
                        component_ok = True
                    else:
                        component_ok = False

                    # Add messages to test result
                    messages = []
                    if component['messages']:
                        if component['messages']:
                            messages = component['messages']

                    tap_component = TapTest(component['name'], component_ok,
                                            messages)
                    tap_plugin.add_subtest(tap_component)

    return tap.dump()

if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright (C) 2007-2015, GoodData(R) Corporation. All rights reserved
"""
Module providing base http server for smokerd REST API
"""

from flask import Flask, request, make_response
from flask.ext.restful import Api, Resource, abort
import json
import multiprocessing
import setproctitle
import socket

from smoker.server import exceptions

# need to keep the daemon instance and common functions at module level since
# there's no other way how to pass the to Flask_restful class methods
smokerd = None


def next_run_iso_format(next_run):
    """
    Convert next run timestamp object to the ISO format
    """
    if isinstance(next_run, bool):
        next_run = None
    else:
        next_run = next_run.isoformat()


def standardized_api_list(component):
    """
    Convert result dict to list just to have standardized API
    """
    keyword = 'componentResults'

    if not component:
        return component
    if not component[keyword]:
        return component

    # Remove reference, because we don't want to modify
    # Plugin object's structures
    results = dict(component)
    results[keyword] = []
    for key, value in component[keyword].iteritems():
        value['name'] = key
        results[keyword].append({'componentResult': value})

    return results


def print_plugin(name, forced=False):
    """
    Print information about a plugin

    :param name: name of the plugin
    :type name: string
    :param forced: use forced_results instead of last_results
    :type forced: bool
    """

    plugin = smokerd.pluginmgr.get_plugin(name)
    plugin.collect_new_result()

    # Format plugin result
    plugin_result = {
        'lastResult': standardized_api_list(plugin.get_last_result()),
        'links': {
            'self': '/plugins/%s' % name,
        },
        'name': name,
        'nextRun': next_run_iso_format(plugin.next_run),
        'parameters': plugin.params
    }

    if forced:
        if not plugin.forced_result:
            raise exceptions.InProgress
        plugin_result['forcedResult'] = plugin.forced_result
    return {'plugin': plugin_result}


def print_plugins(plugins, forced=False):
    """
    Print information about set of plugins

    :param plugins: list of plugin names
    :type plugins: list of strings
    :param forced: use forced_results instead of last_results
    :type forced: bool
    """
    plugins_result = []

    for plugin in plugins:
        plugins_result.append(print_plugin(plugin, forced))

    return {'plugins': {'items': plugins_result}}


def get_plugin_history(name):
    """
    Get history of results for single plugin

    :param name: name of the plugin
    :type name: string
    """
    plugin = smokerd.pluginmgr.get_plugin(name)
    results = []

    for res in plugin.result:
        res = standardized_api_list(res)
        results.append({'result': res})

    return results


def print_in_progress(id):
    """
    Format json info about process in progress

    :param id: process identifier
    :type id: int
    """
    location = '/processes/%d' % id
    data = {
        'asyncTask': {
            'link': {
                'poll': location
            }
        }
    }

    # need to create response manually in orted to have custom status code
    response = make_response(json.dumps(data, indent=2))
    response.status = 'Accepted'
    response.status_code = 202
    response.headers['Location'] = location
    response.headers['content-type'] = 'application/json'
    return response


class About(Resource):
    """
    Print the basic usage
    """
    def get(self):
        return {
            'about': {
                'host': socket.gethostname(),
                'title': 'Smoker daemon API',
                'links': [
                    {
                        'rel': 'plugins',
                        'href': '/plugins',
                        'methods': 'GET',
                        'title': 'Show details about all plugins'
                    },
                    {
                        'rel': 'processes',
                        'href': '/processes',
                        'methods': 'GET, POST',
                        'title': 'Force plugin run'
                    }
                ]
            }
        }


class Plugins(Resource):
    def get(self):
        """
        Print overview of all plugins
        """
        return print_plugins(smokerd.pluginmgr.get_plugins().keys())


class Plugin(Resource):
    def get(self, name):
        """
        Print a single plugin with history of results

        :param name: name of the plugin
        :type name: string
        """
        try:
            plugin = print_plugin(name)
        except exceptions.NoSuchPlugin as e:
            abort(404, message=e.message)
        history = get_plugin_history(name)
        plugin['results'] = history

        return plugin


class Processes(Resource):
    """
    Create or get process
    """
    def get(self):
        result = []
        processes = smokerd.pluginmgr.get_process_list()
        # we index processes from 1, having dummy on position 0
        for id in range(1, len(processes)):
            process = processes[id]
            plugins = []
            for plugin in process['plugins']:
                plugins.append(plugin.name)

            result.append(
                {
                    'href': 'processes/%d' % id,
                    'plugins': plugins
                }
            )
        return result

    def post(self):
        example = {}
        example['example_input'] = {
            'process': {
                'plugins': '[STRING] | NULL',
                'filter': '{STRING : STRING} | NULL'
            }
        }
        example['note'] = (
            'filter is optional key : value pair of plugin parameters '
            'to filter')

        definition = request.get_json(force=True)

        if (not definition or 'process' not in definition
                or not definition['process']):
            abort(400, **example)

        if 'plugins' in definition['process']:
            plugins = definition['process']['plugins']
        else:
            plugins = None

        if 'filter' in definition['process']:
            filter = definition['process']['filter']
        else:
            filter = None

        # If plugins and filter are empty, report bad request
        if not plugins and not filter:
            example['message'] = 'Plugin names or filter have to be set'
            abort(400, **example)

        # Validate input
        if plugins and not isinstance(plugins, list):
            example['message'] = 'Element plugins have to be list'
            abort(400, **example)

        if filter and not isinstance(filter, dict):
            example['message'] = 'Element filter have to be dictionary'
            abort(400, **example)

        try:
            id = smokerd.pluginmgr.add_process(plugins, filter)
        except Exception as e:
            abort(500, message=e.message)

        return print_in_progress(id)


class Process(Resource):
    def get(self, id):
        """
        Get single process result

        :param id: process identifier
        :type id: int
        """
        try:
            if id < 1:
                raise IndexError
            process = smokerd.pluginmgr.get_process(int(id))
        except IndexError:
            abort(404, message='Process id %s not found' % id)

        plugins = []
        for plugin in process['plugins']:
            plugins.append(plugin.name)
            smokerd.pluginmgr.get_plugin(plugin.name).collect_new_result()

        try:
            return print_plugins(plugins, forced=True)
        except exceptions.InProgress:
            return print_in_progress(id)


class RestServer(multiprocessing.Process):
    def __init__(self, host, port, smoker_daemon):
        """
        :param host: host to bind the server to"
        :type host: string
        :param port: port to bind the server to"
        :type port: int
        :param smoker_daemon: instance of the smoker daemon
        :type smoker_daemon: smokerd.Smokerd
        """
        self.host = host
        self.port = port
        self.app = Flask(__name__)

        self.api = Api(self.app)
        self.api.add_resource(About, '/')
        self.api.add_resource(Plugins, '/plugins', '/plugins/')
        self.api.add_resource(Plugin, '/plugins/<string:name>',
                              '/plugins/<string:name>/')
        self.api.add_resource(Processes, '/processes', '/processes/')
        self.api.add_resource(Process, '/processes/<int:id>',
                              '/processes/<int:id>/')

        global smokerd
        smokerd = smoker_daemon

        super(RestServer, self).__init__()

    def run(self):
        setproctitle.setproctitle('smokerd rest api server')
        self.app.run(self.host, self.port)



# -*- coding: utf-8 -*-
# Copyright © 2007-2013, All rights reserved. GoodData® Corporation, http://gooddata.com

__author__ = "miroslav.hedl@gooddata.com"
__maintainer__ = __author__

# xml schema for jenkins junit xml format is here:
# https://svn.jenkins-ci.org/trunk/hudson/dtkit/dtkit-format/dtkit-junit-model/src/main/resources/com/thalesgroup/dtkit/junit/model/xsd/junit-4.xsd


import itertools
import collections
import string


################################################################################
##################################   xml  part   ###############################
################################################################################
class XmlBuilder(object):
    '''
    Template for building basic XML structure.

    Treat properties as xml elements, and kwargs like element attributes.

    :attribute _tag_name: holds tag name for given element instance
    :type tag_name:  string or None
    :attribute _children: holds tag name for given element
    :type _children: list of string or XmlBuilder instances
    :attribute _fields: embed all field names as attributes of xml element
                        (if field value is not None)
    :type _fields: list of tuples (pairs: field name, field value)
    :attribute _subst: if True, convert all bounded names to values;
                            leave template as-is otherwise
    :type _subst: bool
    :attribute _indent: fill beginning of lines with this string (repeat)
    :type _indent: str
    :attribute _crlf: for readabable xml leave as is (for compact xml, set :param:`indent` and :param:`crlf` to empty string)
    :type _crlf: str


    Examples:

    >>> xml = XmlBuilder('xml', subst=False)
    >>> xml.head.title <= "$title"
    >>> xml.color.rgb <= "$bg_color"
    >>> xml.body(id=123, bg_color="$bg_color") <= "I am ${name}!"
    >>> R, G, B = 15, 31, 63
    >>> print xml.dump(bg_color='${R:x}{G:x}{B:x}'.format(R=R, G=G, B=B), name='bingo', title="Simple Example")
    <xml>
      <head>
        <title>$title</title></head>
      <color>
        <rgb>$bg_color</rgb></color>
      <body bg_color="$bg_color" id="123">I am ${name}!</body></xml>

    >>> xml = XmlBuilder('xml')                   # subst=True is default
    >>> xml.head.title <= "$title"
    >>> xml.color.rgb <= "$bg_color"
    >>> xml.body(id=123, bg_color="$bg_color") <= "I am ${name}!"
    >>> R, G, B = 15, 31, 63
    >>> print xml.dump(bg_color='${R:x}{G:x}{B:x}'.format(R=R, G=G, B=B), name='dingo', title="Simple Example")
    <xml>
      <head>
        <title>Simple Example</title></head>
      <color>
        <rgb>$f1f3f</rgb></color>
      <body bg_color="$f1f3f" id="123">I am dingo!</body></xml>

    >>> nested = XmlBuilder('nested-struct')
    >>> (nested.first(name='$name 1', custom_dict={'system-out': "INFO MESSAGE..."})
     >>> <= XmlBuilder().nested(name='$name 2', custom_dict={'system-err': "ERROR MESSAGE..."})
     >>> <= XmlBuilder().other_nested(name='$name 3', custom_dict={'system-err': "ERROR MESSAGE..."})
     >>> <= XmlBuilder().yet_another_nested(name='$name 4', custom_dict={'system-err': "ERROR MESSAGE..."}))
    >>> print nested.dump(name='Case')
    <nested-struct>
      <first system-out="INFO MESSAGE..." name="Case 1">
        <nested system-err="ERROR MESSAGE..." name="Case 2">
          <other_nested system-err="ERROR MESSAGE..." name="Case 3">
            <yet_another_nested system-err="ERROR MESSAGE..." name="Case 4"></yet_another_nested></other_nested></nested></first></nested-struct>
    '''

    def __init__(self, tag_name=None, subst=True, indent='  ', crlf='\n'):
        '''
        Create new xml node.

        :param str? tag_name: If present, element has tag name and behaves normally
        :param bool subst: Do substitute unbounded names (in format '$name' or x-${Name}-y)
        :param str indent: Delimiter for indenting
        :param str crlf: Delimiter for line-breaks
        :rtype: XmlBuilder
        :return: instance that can be used as: callable, smart getattr, in with statement
        '''

        self._tag_name = tag_name
        self._children = []
        self._fields = []
        self._subst = subst
        self._indent = indent
        self._crlf = crlf

    ## special methods
    def __getattr__(self, tag_name):

        if tag_name.startswith('_'):
            return self.__dict__.get(tag_name)
        new = self.__class__(tag_name=tag_name,
                             subst=self._subst,
                             indent=self._indent,
                             crlf=self._crlf,
                             )
        self._children.append(new)
        return new

    def __setattr__(self, name, value):
        if name.startswith('_'):
            self.__dict__[name] = value
        else:
            self._fields.append((name, value))

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        return False

    def __call__(self, custom_dict=None, **kwargs):
        '''
        Applies xml attributes to element.
        :param dict custom_dict: for values that not match python variables (like: 'system-err')
        :param dict kwargs: kwargs will override values in custom dict
        '''
        if custom_dict:
            for key, val in custom_dict.iteritems():
                self._fields.append((key, val))
        for key, val in kwargs.iteritems():
            self._fields.append((key, val))
        return self

    ## hidden helper methods
    def _sub(self, text, **kwargs):
        '''
        Evaluate text in environment of kwargs (do nothing if ..py:attr:`_subst` is False)
        '''
        if not self._subst:
            return text
        return string.Template(text).safe_substitute(**kwargs)

    def _open_tag(self, indent_lvl=0, **kwargs):
        '''
        Applies tuple fields (name, value) to xml.
        Both items must be not None to show in attribute list.
        '''
        return self._sub(
            self._crlf + "{i}<{name}".format(i=self._indent * indent_lvl,
                                             name=self._tag_name)
            + ''.join(' {name}="{val}"'.format(name=name, val=val)
                      for (name, val) in self._fields
                      if val is not None)
            + '>', **kwargs)

    def _close_tag(self, indent_lvl=0, **kwargs):
        '''Will close matching tag. Will nest to the right (better readability)'''
        return "</{name}>".format(name=self._tag_name)

    ## allows '<=' to add children
    def __le__(self, other):
        self._children.append(other)
        return self

    ## public methods
    def innerText(self, child):
        '''
        Will add text or another (arbitrary complex) XmlBuilder as children of element.
        '''
        self._children.append(child)
        return self

    def dump(self, indent_lvl=0, **kwargs):
        '''
        Recursively converts self and all children to xml.

        :param int indent_lvl: starting indentation level (less or eq then 0 is nothing)
        :param dict kwargs: provide means to influence value of unbounded template fields
        :rtype: string
        :return: formatted string in xml format
        '''
        dumped_children = []
        for child in itertools.ifilter(None, self._children):
            if isinstance(child, XmlBuilder):
                dumped_children.append(child.dump(indent_lvl=(indent_lvl + 1),
                                                  **kwargs))
            else:
                dumped_children.append("{child}".format(child=self._sub(child,
                                                                        **kwargs)))
        if self._tag_name:
            return (self._open_tag(indent_lvl, **kwargs)
                    + ''.join(dumped_children)
                    + self._close_tag(indent_lvl, **kwargs))
        return ''.join(dumped_children)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2012, GoodData(R) Corporation. All rights reserved

"""
Check file system mounts
Checks mount points for access and read/write

Parameters:
    Ignore  - regular expression for filtering ignored mount points (matches from the beginning of string)
    Types   - list of included filesystem types (default ext[234], fuse.glusterfs, nfs, nfs4, xfs, btrfs, reiserfs)
    Mounts  - list of mounts to check (default try to check all)
"""

import os
import random
import re
import logging

from smoker.server.plugins import BasePlugin

lg = logging.getLogger('smokerd.plugin.fsmount')

class Plugin(BasePlugin):
    def run(self):
        # Get parameters
        ignore = self.plugin.get_param('Ignore', default=None)
        types  = self.plugin.get_param('Types',
            default=['ext2', 'ext3', 'ext4', 'fuse.glusterfs', 'nfs', 'nfs4', 'xfs', 'btrfs', 'reiserfs'])
        check_mounts = self.plugin.get_param('Mounts', default=None)

        # Get all mounts
        try:
            mounts = self.get_mounts(ignore=ignore, types=types, mounts=check_mounts)
        except Exception as e:
            self.result.set_status('ERROR')
            self.result.add_error("Getting list of mounts failed: %s" % e)
            return self.result

        # No mounts found
        if not mounts:
            self.result.set_status('WARN')
            self.result.add_warn('No mounts found')
            return self.result

        for path, mount in mounts.iteritems():
            try:
                res = self.check_mount(path, mount)
                self.result.add_component(path, res['state'], **res['messages'])
            except Exception as e:
                self.result.add_component(path, 'ERROR', error=[str(e)])
                lg.exception(e)

        self.result.set_status()
        return self.result

    def check_mount(self, path, mount):
        """
        Check given mount
        """
        result = {
            'state' : 'OK',
            'messages': {
                'info'  : [],
                'error' : [],
                'warn'  : [],
            },
        }

        # Access check
        try:
            test = self.check_access(path)
            result['state'] = test['state']

            # Append messages
            for type, messages in test['messages'].iteritems():
                for msg in messages:
                    msg = 'Access: %s' % msg
                    result['messages'][type].append(msg)
        except Exception as e:
            result['state'] = 'ERROR'
            result['messages']['error'].append('Access: %s' % e)

        # Read/write check
        try:
            test = self.check_readwrite(path)

            # Update state
            if test['state'] == 'ERROR':
                result['state'] = 'ERROR'
            elif test['state'] == 'WARN' and result['state'] == 'OK':
                result['state'] = 'WARN'

            # Append messages
            for type, messages in test['messages'].iteritems():
                for msg in messages:
                    msg = 'Read/Write: %s' % msg
                    result['messages'][type].append(msg)
        except Exception as e:
            result['state'] = 'ERROR'
            result['messages']['error'].append('Read/Write: %s' % e)

        return result

    def check_access(self, path):
        """
        Check mount access
        """
        result = {
            'state' : 'OK',
            'messages': {
                'info'  : [],
                'error' : [],
                'warn'  : [],
            },
        }

        listing = os.listdir(path)
        result['messages']['info'].append("listed %d items in directory" % len(listing))

        return result

    def check_readwrite(self, path):
        """
        Check mount write and read
        """
        result = {
            'state' : 'OK',
            'messages': {
                'info'  : [],
                'error' : [],
                'warn'  : [],
            },
        }

        filename = random.randrange(0, 99999999, 8)
        filepath = ('%s/%s-smoker.tmp' % (path, filename)).replace('//', '/')

        # Write test
        try:
            fh = open(filepath, 'w')
            fh.write(str(filename))
            fh.flush()
            fh.close()
            result['messages']['info'].append('written string %s' % filename)
        except Exception as e:
            result['state'] = 'ERROR'
            result['messages']['error'].append('file write failed: %s' %e)
            return result

        # Read test
        try:
            fh = open(filepath, 'r')
            string = fh.read()
            fh.close()
            result['messages']['info'].append('read string %s' % string)
        except Exception as e:
            result['state'] = 'ERROR'
            result['messages']['error'].append('file read failed: %s' %e)
            pass
        finally:
            # Delete file
            try:
                os.unlink(filepath)
                result['messages']['info'].append('deleted file %s' % filepath)
            except Exception as e:
                result['state'] = 'ERROR'
                result['messages']['error'].append('file deletion failed: %s' %e)
                pass

        return result

    def get_mounts(self, ignore=None, types=[], mounts=[]):
        """
        Get mounted filesystems
        ignore - regular expressions for ignoring mount points (matches from the beginning of string)
        types  - filesystem types to look at
        mounts - return only specific mounts
        """
        result = {}
        with open('/proc/mounts', 'r') as fh:
            for line in fh:
                line = line.replace('\n', '')
                mount = line.split(' ')

                if mounts and mount[1] not in mounts:
                    continue
                elif types and mount[2] not in types:
                    continue
                elif ignore and re.match(ignore, mount[1]):
                    # Ignore mount point that matches regular expression
                    lg.debug('Ignoring mount point %s' % mount[1])
                    continue
                # do not check netapp snapshots (RO filesystems)
                elif '.snapshot' in mount[1]:
                    continue
                else:
                    result[mount[1]] = {
                        'device' : mount[0],
                        'mount'  : mount[1],
                        'type'   : mount[2],
                        'options': mount[3].split(','),
                        'dump'   : mount[4],
                        'pass'   : mount[5]
                    }

        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2012, GoodData(R) Corporation. All rights reserved

"""
Module providing exceptions for smokerd
"""

# PluginManager exceptions
class TemplateNotFound(Exception):
    """
    Can't find template
    """
    pass

class NoTemplatesConfigured(Exception):
    """
    There are no configured templates
    """
    pass

class BasePluginTemplateNotFound(Exception):
    """
    BasePlugin template is not found
    """
    pass

class NoRunningPlugins(Exception):
    """
    No plugins loaded
    """
    pass

class ActionNotFound(Exception):
    """
    Can't find action
    """
    pass

class NoActionsConfigured(Exception):
    """
    There are no configured actions
    """
    pass

class InvalidArgument(Exception):
    """
    Invalid argument
    """
    pass

class ValidationError(Exception):
    """
    Error during validation
    """
    pass

class NoSuchPlugin(Exception):
    """
    Plugin does not exists
    """
    pass

class NoPluginsFound(Exception):
    """
    No plugins was found by given name(s) or filter
    """

# Plugin exceptions
class PluginExecutionError(Exception):
    """
    Plugin execution failed
    eg. command not found, etc.
    """
    pass

class PluginExecutionTimeout(Exception):
    """
    Plugin execution timeouted
    """
    pass

class PluginMalformedOutput(Exception):
    """
    Plugin output was not recognized
    eg. malformed JSON, missing status in output
    """
    pass

class InvalidConfiguration(Exception):
    """
    Not valid configuration parameters
    or their combination supplied
    """
    pass

# Httpserver exceptions
class InProgress(Exception):
    """
    Process run is still in progress
    """
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2012, GoodData(R) Corporation. All rights reserved

import glob
import logging
import os
import signal
import sys
import time
import yaml

from smoker.server.plugins import PluginManager
from smoker.server.restserver import RestServer

lg = logging.getLogger('smokerd.daemon')


class Smokerd(object):
    """
    Main entrance for smokerd
    """

    conf = {}

    # ThreadedHTTPServer instance
    server = None

    # PluginManager instance
    pluginmgr = None

    def __init__(self, **kwargs):
        """
        Initialize smokerd
         * load submitted arguments
         * load config
         * catch SIGINT/SIGTERM
        """
        # Load user-supplied arguments and config file
        # args have to be dictionary
        self.conf = kwargs

        if 'config' not in kwargs:
            lg.info("Config argument not submitted, default config file will be used!")
            self.conf['config'] = '/etc/smokerd/smokerd.yaml'

        self.conf_dirs = [os.path.dirname(self.conf['config'])]

        self._load_config()

    def _yaml_include(self, loader, node):
        """
        Include another yaml file from main file
        This is usually done by registering !include tag
        """
        filepath = node.value
        if not os.path.exists(filepath):
            for dir in self.conf_dirs:
                filepath = os.path.join(dir, node.value)
                if os.path.exists(filepath):
                    break

        self.conf_dirs.append(os.path.dirname(filepath))
        try:
            with open(filepath, 'r') as inputfile:
                return yaml.load(inputfile)
        except IOError as e:
            lg.error("Can't include config file %s: %s" % (filepath, e))
            raise

    def _yaml_include_dir(self, loader, node):
        """
        Include another yaml file from main file
        This is usually done by registering !include tag
        """
        if not os.path.exists(node.value):
            return

        yamls = glob.glob(os.path.join(node.value, '*.yaml'))
        if not yamls:
            return

        content = '\n'

        for file in yamls:
            plugin, _ = os.path.splitext(os.path.basename(file))
            content += '    %s: !include %s\n' % (plugin, file)

        return yaml.load(content)

    def _load_config(self):
        """
        Load specified config file
        """
        try:
            with open(self.conf['config'], 'r') as fp:
                config = fp.read()
        except IOError as e:
            lg.error("Can't read config file %s: %s" % (self.conf['config'], e))
            raise

        # Register include constructors
        yaml.add_constructor('!include_dir', self._yaml_include_dir)
        yaml.add_constructor('!include', self._yaml_include)

        try:
            conf = yaml.load(config)
        except Exception as e:
            lg.error("Can't parse config file %s: %s" % (self.conf['config'], e))
            raise
        finally:
            fp.close()

        # Store parameters but don't overwite
        # those submitted by command line
        for key, value in conf.iteritems():
            if self.conf.has_key(key):
                # User has submitted own parameter,
                # use that instead of config one
                lg.debug("Using parameter %s from user, ignoring config file value" % key)
            else:
                self.conf[key] = value

    def run(self):
        """
        Run daemon
         * change effective uid/gid
         * start thread for each check
         * start webserver
        """
        lg.info("Starting daemon")

        # Change effective UID/GID
        if self.conf.has_key('uid') and self.conf.has_key('gid'):
            if os.geteuid != self.conf['uid'] and os.getegid != self.conf['gid']:
                try:
                    os.setegid(self.conf['gid'])
                    os.seteuid(self.conf['uid'])
                except TypeError as e:
                    lg.error("Config parameters uid/gid have to be integers: %s" % e)
                except OSError as e:
                    lg.error("Can't switch effective UID/GID to %s/%s: %s" % (self.conf['uid'], self.conf['gid'], e))
                    lg.exception(e)
                    self._shutdown(exitcode=1)
        else:
            lg.info("Not changing effective UID/GID, keeping %s/%s" % (os.geteuid(), os.getegid()))

        if not isinstance(self.conf['bind_port'], int):
            lg.error("Config parameter bind_port has to be integer")

        # Initialize plugin manager
        config = {}

        for key in ['plugins', 'templates', 'actions']:
            try:
                config[key] = self.conf[key]
            except KeyError as e:
                lg.warn("Config section not found: %s" % e)

        # Check we have some plugins configured
        if not config['plugins']:
            lg.error('No configured plugins')
            self._shutdown(exitcode=1)

        if 'nr_concurrent_plugins' in self.conf:
            config['semaphore_count'] = self.conf['nr_concurrent_plugins']

        try:
            self.pluginmgr = PluginManager(**config)
        except Exception as e:
            lg.error("Can't initialize PluginManager")
            lg.exception(e)
            self._shutdown(exitcode=1)

        lg.info("Starting webserver on %(bind_host)s:%(bind_port)s"
                % self.conf)
        try:
            self.server = RestServer(self.conf['bind_host'],
                                     self.conf['bind_port'], self)
            self.server.start()
        except Exception as e:
            lg.error("Can't start HTTP server: %s" % e)
            lg.exception(e)
            self._shutdown(exitcode=1)

        # Catch SIGINT and SIGTERM if supported
        if hasattr(signal, 'SIGINT'):
            signal.signal(signal.SIGINT, self._shutdown)

        if hasattr(signal, 'SIGTERM'):
            signal.signal(signal.SIGTERM, self._shutdown)
        # API server loop now runs in a separate process and we don't want
        # to terminate to keep an instance of pluginmanager

        self._watchdog()

    def _watchdog(self):
        lg.debug("Starting the smoker watchdog")

        while True:
            if not self.server.is_alive():
                lg.error("REST API server is dead")
                self._restart_api_server()
                lg.info("restarted the REST API server")

            # Take care of execution of the timed plugins
            self.pluginmgr.run_plugins_with_interval()
            self.pluginmgr.join_timed_plugin_workers()

            time.sleep(5)

    def _restart_api_server(self):
        self.server.terminate()
        self.server.join()
        self.server = RestServer(self.conf['bind_host'], self.conf['bind_port'], self)
        self.server.start()

    def stop(self):
        """
        Kill running daemon

        Use sys.exit() here instead of self._shutdown(), because it's executed from separate process
        """
        if not os.path.isfile(self.conf['pidfile']):
            lg.error("PID file doesn't exist! Daemon not running?")
            sys.exit(1)
        fp = open(self.conf['pidfile'], 'r')
        pid = fp.read()
        fp.close()
        lg.info("Killing process %s" % pid)
        os.kill(int(pid), signal.SIGTERM)

    def daemonize(self):
        """
        Daemonize and run daemon
        Use double-fork magic to do that

        Use sys.exit() here instead of self._shutdown() because we don't have running
        daemon to shutdown in this function
        """
        # Check PID file
        if os.path.isfile(self.conf['pidfile']):
            lg.error("PID file %s already exists" % self.conf['pidfile'])
            sys.exit(1)

        # Unix double-fork magic
        pid = os.fork()
        if pid:
            sys.exit(0)

        os.chdir('/')
        os.setsid()
        os.umask(0)

        pid = os.fork()
        if pid:
            sys.exit(0)

        # Ensure that directories for stdout and stderr logging
        # exists or create them
        for log in ['stdout', 'stderr', 'stdin']:
            path = os.path.dirname(self.conf[log])
            if not os.path.exists(path):
                os.mkdir(path)

        # Redirect standard I/O
        sys.stdout.flush()
        sys.stderr.flush()
        try: 
            si = file(self.conf['stdin'], 'r')
            so = file(self.conf['stdout'], 'a+')
            se = file(self.conf['stderr'], 'a+', 0)
        except Exception as e:
            lg.error("Can't open configured output: %s" % e)
            lg.exception(e)
            sys.exit(1)
        os.dup2(si.fileno(), sys.stdin.fileno())
        os.dup2(so.fileno(), sys.stdout.fileno())
        os.dup2(se.fileno(), sys.stderr.fileno())

        # Save PID into pidfile
        try:
            fh = open(self.conf['pidfile'], 'w')
            fh.write(str(os.getpid()))
            fh.flush()
            fh.close()
        except Exception as e:
            lg.error("Can't write PID into pidfile: %s" % e)
            sys.exit(1)

        try:
            self.run()
        except Exception as e:
            # On exception, try to shutdown and log
            self._shutdown(exitcode=1)
            lg.exception(e)

    def _shutdown(self, signum=None, frame=None, exitcode=0, exception=False):
        """
        Shutdown smoker daemon (internal use)
        exitcode - exit code number (default 0)
        signum, frame - used by signal handler
        exception - if True, raise on exception, otherwise just log it and pass
        """
        # Ignore if we are already stopping
        if self.pluginmgr and self.pluginmgr.stopping:
            return

        lg.info("Shutting down")
        try:
            # Shutdown webserver
            if self.server:
                self.server.terminate()
                self.server.join()

            # Shutdown pluginmanager and all plugins
            if self.pluginmgr:
                self.pluginmgr.stop()

            # Remove PID file if exists
            if os.path.isfile(self.conf['pidfile']):
                os.remove(self.conf['pidfile'])
        except Exception as e:
            lg.exception(e)
            if exception:
                raise

        sys.exit(exitcode)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2013, GoodData(R) Corporation. All rights reserved

"""
Module for various command executions
"""

import subprocess
import threading
import os
import psutil
import atexit
import datetime
import logging
import time
lg = logging.getLogger(__name__)

def execute(command, timeout=None, **kwargs):
    """
    Execute command, wrapper for Command class

    :param command: list for non-shell execution, string for shell execution
    :param timeout: timeout in seconds
    :param kwargs: keyword arguments to pass to subprocess.Popen

    :rtype: tuple (stdout, stderr, retval)
    """
    cmd = Command(command, **kwargs)
    return cmd.run(timeout)

def signal_ptree(pid, signal=15):
    """
    Send signal to whole process tree
    By default send SIGTERM (15).
    If process doesn't exist, just pass

    :param pid: process id
    :param signal: signal number, send SIGTERM (15) by default
    """
    try:
        process = psutil.Process(pid)
    except psutil.NoSuchProcess:
        # Process could be already dead, just skip killing children
        return

    # Get children process tree (psutil 0.4.1 doesn't support recursive=True)
    children = get_ptree(process)

    lg.info('Sending signal to process tree: signal=%s pid=%s process=%s children=%s' % (signal, process.pid, process.name, len(children)))

    if children:
        # Children have to be from bottom to top list
        for child in children:
            try:
                lg.info('Sending signal to child process: signal=%s pid=%s process=%s' % (signal, child.pid, child.name))
                os.kill(child.pid, signal)
            except OSError as e:
                if e.errno == 3:
                    # No such process - it's ok, it could be dead already
                    lg.debug('Children process does not exist: pid=%s process=%s' % (child.pid, child.name))
                    continue

    # Kill parent
    try:
        lg.info('Sending signal to parent process: signal=%s pid=%s process=%s' % (signal, process.pid, process.name))
        os.kill(process.pid, signal)
    except OSError as e:
        if e.errno == 3:
            # No such process - it's ok, it could die with it's children
            lg.debug('Parent process does not exist: pid=%s process=%s' % (process.pid, process.name))
            pass

def get_ptree(process):
    """
    Get process children recursive.
    Used for compatibility with psutil 0.4.1, newer versions supports recursive=True parameter.
    Given process isn't included in returned list.
    Process tree list is reversed, so first are children from bottom to top.

    :param process: psutil.Process instance or uid
    :rtype: list (psutil.Process)
    """
    if not isinstance(process, psutil.Process):
        process = psutil.Process(process)

    result = []
    children = process.get_children()
    if children:
        for child in children:
            if child.get_children():
                result.extend(get_ptree(child))
                result.append(child)
            else:
                result.append(child)
    return result

def _proc_cleanup(pid):
    """
    Try to cleanup process tree

    :param pid: process id
    """
    if pid:
        try:
            process = psutil.Process(pid)
        except psutil.NoSuchProcess:
            return

        signal_ptree(process)
        if process.is_running():
            # Still running - wait 1 second before sending SIGKILL
            time.sleep(1)
            signal_ptree(process, 9)

def _register_cleanup(pid):
    """
    Register cleanup function for given process id

    :param pid: process id
    """
    lg.debug("Registering cleanup for pid %s" % pid)
    atexit.register(_proc_cleanup, pid)

def _unregister_cleanup(pid):
    """
    Unregister cleanup function for given process id

    :param pid: process id
    """
    lg.debug("Unregistering cleanup for pid %s" % pid)

    # Newer atexit has unregister, but we want to be compatible
    for handler in atexit._exithandlers:
        (func, args, kwargs) = handler
        if func == _proc_cleanup and args == (pid,):
            atexit._exithandlers.remove(handler)

class Command(object):
    """
    Class for command executions
    """
    def __init__(self, command, **kwargs):
        """
        Initialize instance

        :param command: list for non-shell execution, string for shell execution
        :param kwargs: keyword arguments to pass to subprocess.Popen
        """
        self.command = command

        self.process = None

        self.stdout = None
        self.stderr = None
        self.returncode = None

        self._exception = None

        # Default arguments
        popen_args = {
            'stdout': subprocess.PIPE,
            'stderr': subprocess.PIPE,
            'bufsize': 0
        }

        # We want to pass shell=True argument if we have string command
        if isinstance(command, basestring):
            popen_args['shell'] = True

        # Merge our arguments and supplied ones in kwargs
        self.kwargs = dict(popen_args, **kwargs)

    def __repr__(self):
        """
        Instance name
        """
        return '<Command \'%s\'>' % self.command

    def run(self, timeout=None, timeout_sigterm=3, timeout_sigkill=5):
        """
        Run command with given timeout.
        Return tuple of stdout, stderr strings and retval integer.

        :param timeout: if command doesn't exit in given timeout, kill the process (default no timeout)
        :param timeout_sigterm: wait approximately given seconds after sending SIGTERM before sending SIGKILL (default 3)
        :param timeout_sigkill: wait approximately given seconds after sending SIGKILL before considering thread as deadlocked (default 5)

        :rtype: tuple (stdout, stderr, retval)
        """
        def target():
            """
            Thread target function
            """
            try:
                self.process = subprocess.Popen(self.command, **self.kwargs)
                # Register cleanup function to avoid running processes after program exit
                _register_cleanup(self.process.pid)
                self.stdout, self.stderr = self.process.communicate()

                # Remove unwanted leading/trailing whitespaces from output
                # Force stdout/stderr to be string if it's empty
                self.stdout = self.stdout.strip() if self.stdout else ''
                self.stderr = self.stderr.strip() if self.stderr else ''

                self.returncode = self.process.returncode
            except Exception as e:
                self._exception = e
                return e

        # Run thread with command and wait
        thread = threading.Thread(target=target)
        lg.debug("Executing command: command='%s' %s"
            % (self.command, ' '.join('%s=%s' % (a, b) for a, b in self.kwargs.iteritems())))
        time_start = datetime.datetime.now()
        thread.start()

        if timeout:
            thread.join(timeout)

            # Thread still alive? Timeout!
            if thread.is_alive():
                # Terminate process and wait 3 seconds
                signal_ptree(self.process.pid)
                thread.join(timeout_sigterm)

                if thread.is_alive():
                    # Thread still alive -> send SIGKILL
                    signal_ptree(self.process.pid, signal=9)
                    thread.join(timeout_sigkill)

                    if thread.is_alive():
                        # Thread still alive -> deadlock
                        # Unregister cleanup function in case that process would die to avoid killing re-used PID
                        _unregister_cleanup(self.process.pid)
                        raise ThreadDeadlock("Process %s deadlocked thread %s" % (self.process.pid, thread.name))
                # Process is no running, unregister cleanup and raise Timeout exception
                _unregister_cleanup(self.process.pid)
                raise ExecutionTimeout("Execution timeout after %s seconds" % timeout)
        else:
            # No timeout applied.. only insane people should do this.
            thread.join()

        # Handle exception from thread
        if self._exception:
            # It means that process is not running, so unregister cleanup and re-raise exception
            _unregister_cleanup(self.process.pid)
            raise self._exception

        lg.debug("Command execution done: time=%s returncode=%s" %
                 ((datetime.datetime.now() - time_start).seconds, self.returncode))

        # We are successfully done, unregister cleanup to avoid killing re-used PID uppon server shutdown
        _unregister_cleanup(self.process.pid)
        return (self.stdout, self.stderr, self.returncode)

## Exceptions
class ExecutionTimeout(Exception):
    """
    Raise timeout exception
    """
    pass

class ThreadDeadlock(Exception):
    """
    Process can't be killed, caused deadlock of executing thread
    """
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2013, GoodData(R) Corporation. All rights reserved

"""
ProgressBar util

Using by with syntax is preferred, otherwise you have to call progress.stop() for cleanup.

Typical usage (with pool of threads):

    with ProgressBar(len(pool)) as progress:
        while progress.get_left():
            done = 0
            for t in pool:
                if not t.isAlive():
                    done += 1
            progress.set_done(done)


Or when you are having pool of threads, you can use following shorter syntax:

    with ProgressBar(len(pool)) as progress:
        progress.wait_pool(pool)

And it's always good idea to fallback to non-progress wait (catch NonInteractiveError exception)

from util.progressbar import ProgressBar, NonInteractiveError
try:
    progress = ProgressBar(len(pool))
except NonInteractiveError as e:
    print e
    # fallback to non-progress wait
"""

import sys
import numbers
import threading
import time

from smoker.util import console

### Exceptions
class InvalidAnimationError(Exception):
    pass

class NonInteractiveError(Exception):
    pass

### Main class
class ProgressBar(threading.Thread, object):
    """
    Progressbar instance
    """
    elements = None
    template = None
    speed    = 0.08

    items_count = None
    items_done  = None

    def __init__(self, items, speed=None, template=None, elements=None, no_check_interactive=False):
        """
        Initialize progressbar
        Set parameters, initialize threading.Thread
        """
        assert isinstance(items, numbers.Integral), "Parameter items must be integral number"
        assert (isinstance(speed, numbers.Real) or speed is None), "Parameter speed must be real number"

        if not no_check_interactive:
            if not console.is_interactive_shell():
                raise NonInteractiveError("Shell seems to be non-interactive. Can't use progress bar.")

        self.items_count = items
        self.items_done  = 0

        self.speed = speed or self.speed

        if not elements:
            # Initialize basic elements
            self.elements = {
                'wheel' : WheelElement(self),
                'bar'   : ProgressElement(self),
                'counter' : CounterElement(self),
            }

        if not template:
            # Set basic template
            self.template = '{wheel} {bar}\t{counter}'

        # Run Thread constructor, we want to be daemonic thread
        super(ProgressBar, self).__init__()
        self.daemon = True

    def add_done(self, count=1):
        """
        Add count of items as done
        """
        assert isinstance(count, numbers.Integral), "Count must be integral number"
        self.items_done += count

    def get_left(self):
        """
        Return number of left items
        """
        return self.items_count - self.items_done

    def set_done(self, done=None):
        """
        Set all items as done or specific count
        """
        if done is not None:
            assert isinstance(done, numbers.Integral), "Done count must be integral number"
            self.items_done = done
        else:
            self.items_done = self.items_count

    def run(self):
        """
        Run thread
        While we have undone items, print progressbar
        """
        while self.items_done < self.items_count:
            # Cleanup to fix progress after terminal resize
            try:
                width, height = console.get_terminal_size()
            except IOError:
                # Fallback to default values
                width, height = (80, 37)
            sys.stdout.write('\r%s' % (width * ' '))

            # Write progress
            sys.stdout.write('\r %s' % self.get_progress())
            sys.stdout.flush()
            time.sleep(self.speed)

    def stop(self):
        """
        This function has to be called after progress bar stop to do the cleanup
        """
        print

    def __exit__(self, type, value, traceback):
        """
        This function is called when using with syntax
        """
        self.stop()

    def __enter__(self):
        """
        This function is called when using with syntax
        Returns self
        """
        self.start()
        return self

    def get_progress(self):
        """
        Format and return progress bar
        """
        progress = self.template.format(**self.elements)
        return progress

    def wait_pool(self, pool):
        """
        Accept list pool of threads, watch if they are alive and update progress
        """
        assert isinstance(pool, list), "Parameter pool must be list of Thread objects"

        while self.get_left():
            done = 0
            for t in pool:
                assert isinstance(t, threading.Thread),\
                    "Object in pool must be instance of threading.Thread not %s" % type(t)
                if not t.isAlive():
                    done += 1
            self.set_done(done)
            time.sleep(1)

### Elements
class Element(object):
    """
    Base Element class
    """
    def __init__(self, main):
        """
        Initialize Element
        Store main class instance
        """
        self.main = main

class AnimationElement(Element):
    """
    Base animation element class
    """
    animation = None
    index = None

    def __str__(self):
        """
        Return current state
        """
        # Update state if we are animated element
        if isinstance(self.animation, list):
            index = self.__update_index()
            return str(self.animation[index])
        else:
            raise InvalidAnimationError('Animation has to be list not %s' % type(self.animation))

    def __update_index(self):
        """
        Update animation state
        Return index
        """
        if self.index is None:
            self.index = 0
        else:
            # Code bellow is faster than catching IndexError exception
            self.index += 1
            if len(self.animation) == self.index:
                self.index = 0
        return self.index

class WheelElement(AnimationElement):
    """
    Element that renders spinning wheel
    """
    animation = ['|', '/', '–', '\\']

class ProgressElement(Element):
    """
    Basic progress element
    """
    progress_part  = '='
    progress_start = ''
    progress_end   = '>'
    bar_start = '['
    bar_end   = ']'
    bar_width  = 80

    def __str__(self):
        """
        Render progress bar
        """
        return self.get_bar()

    def get_bar(self):
        """
        calculate and return progress bar string
        """
        # Get terminal size
        try:
            width, height = console.get_terminal_size()
        except IOError:
            # Fallback to default values
            width, height = (80, 37)

        # Bar should be half a terminal width
        self.bar_width = width / 2

        # Calculate step size
        part_size = float(self.bar_width) / float(self.main.items_count)
        bar_size  = int(part_size * self.main.items_done)

        info = {
            'bar_start' : self.bar_start,
            'progress_start' : self.progress_start,
            'bar' : '%s%s' % (bar_size * self.progress_part, self.progress_end),
            'bar_width': self.bar_width,
            'bar_end' : self.bar_end
        }
        render = '{bar_start}{progress_start}{bar:<{bar_width}}{bar_end}'.format(**info)

        return render

class CounterElement(Element):
    """
    Basic counter element
    """
    template = '{done}/{total}'

    def __str__(self):
        counter = self.template.format(**{'done' : self.main.items_done, 'total' : self.main.items_count })
        return counter



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2012, GoodData(R) Corporation. All rights reserved

"""
Initialize GDC logging

Initialize:
import smoker.logger as logger
import logging
logger.init()
lg = logging.getLogger('application')
lg.info("Logger loaded")

Usage:
import logging
lg = logging.getLogger('application')
"""

import logging
import logging.config
import logging.handlers
import os
from smoker.logger.level_handler import LevelHandler

lg = None


def disable_console_logging(logger):
    handlers = logger.handlers
    for handler in handlers:
        if isinstance(handler, logging.StreamHandler):
            logger.removeHandler(handler)
        if logger.parent:
            disable_console_logging(logger.parent)


# Initialize logging
def init(name='', level=logging.WARN, syslog=True, console=True,
         config_file=''):
    global lg

    if os.path.exists(config_file):
        logging.config.fileConfig(config_file)

    lg = logging.getLogger(name)
    lg.setLevel(level)

    if os.path.exists(config_file):
        if not console:
            disable_console_logging(lg)
        return lg

    if console:
        lg_console = LevelHandler()
        lg_console.setFormatter(
            logging.Formatter('%(levelname)s: %(message)s'))

        lg.addHandler(lg_console)

    if syslog:
        lg_syslog = logging.handlers.SysLogHandler(address='/dev/log')
        lg_syslog.setFormatter(
            logging.Formatter('%(name)-9s %(levelname)-8s %(message)s'))

        lg.addHandler(lg_syslog)

    return lg



# -*- coding: utf-8 -*-
# Copyright (C) 2007-2015, GoodData(R) Corporation. All rights reserved
#
# Example plugin:
#
# from smoker.client.plugins import SpecificArgument, HostDiscoveryPluginBase
#
#
# class HostDiscoveryPlugin(HostDiscoveryPluginBase):
#     """
#     This is an example without any real world usability
#     """
#     arguments = [
#         SpecificArgument(
#             '-x',
#             '--example',
#             **{'dest': 'example',
#                'help': 'Example parameter for host discovery'}
#         ),
#         SpecificArgument(
#             '-y',
#             '--example_prefix',
#             **{'dest': 'prefix',
#                'help': 'Another example for host discovery'}
#         )
#     ]
#
#     def get_hosts(self, args):
#         if not args.example:
#             return []
#         if not args.prefix:
#             return [args.example]
#         return ['%s-%s' % (args.prefix, args.example)]


class SpecificArgument(object):
    """
    Argparse argument to be added to the smoker CLI specific to this plugin
    """
    def __init__(self, short, long, **kwargs):
        if short and long:
            self.args = [short, long]
        elif short:
            self.args = [short]
        else:
            self.args = [long]
        self.kwargs = kwargs


class HostDiscoveryPluginBase(object):
    """
    Host discovery plugin interface
    Inherit from this class when creating a host discovery plugin
    """
    # List of Specific Argument instances to be added
    # to argparse.ArgumentParser
    arguments = []

    def get_hosts(self, args):
        """
        Override this method in your plugin

        :return: discovered hosts
        """
        return []



#!/usr/bin/env python
# -*- coding: utf8 -*-
# Copyright (C) 2007-2012, GoodData(R) Corporation. All rights reserved

"""
Plugin to get overall GlusterFS status
This will check only up to 50 gluster peers

Parameters:
    Volume  - volume to run mount/unmount test. If it's not set, mount test won't be executed.
"""

import logging
lg = logging.getLogger('smokerd.plugin.glusterfs')

from smoker.server.plugins import BasePlugin
from xml.dom import minidom
import subprocess
import os
import tempfile
import random
import time

class Plugin(BasePlugin):
    def run(self):
        """
        Main entrance
         * check peers status
         * check volumes status
         * try to mount and unmount volume
        """

        volume = self.plugin.get_param('Volume')

        try:
            self.check_peers()
        except Exception as e:
            self.result.add_component('Peers', 'ERROR', error=["Can't check peers: %s" % e])
            lg.exception(e)

        try:
            self.check_volumes()
        except Exception as e:
            self.result.add_component('Volumes', 'ERROR', error=["Can't check volumes: %s" % e])
            lg.exception(e)

        if volume:
            try:
                self.mount(volume)
            except Exception as e:
                self.result.add_component('Mount', 'ERROR', error=["Can't mount volume: %s" % e])
                lg.exception(e)

        self.result.set_status()
        return self.result

    def check_peers(self):
        """
        Check peers status
        and add component result
        """
        peers = self.getPeersStatus()
        status = 'OK'
        messages = {
            'info' : [],
            'error': [],
            'warn' : [],
        }

        for host, peer in peers.iteritems():
            if peer['connected'] == True:
                messages['info'].append('Peer %s is healthy: %s (Connected)' % (host, peer['status']))
            else:
                messages['error'].append('Peer %s is not healthy: %s (Disconnected)' % (host, peer['status']))
                status = 'ERROR'

        self.result.add_component('Peers', status, **messages)

    def check_volumes(self):
        """
        Check volumes status
        and add component result
        """
        volumes = self.getVolumesStatus()
        status = 'OK'
        messages = {
            'info' : [],
            'error': [],
            'warn' : [],
        }

        # Less than 1 volume?
        if len(volumes) < 1:
            messages['error'].append("No configured volumes found")
            status = 'ERROR'

        # Check status of each volume
        for vol, nodes in volumes.iteritems():
            if nodes['status'] != 1:
                # Get broken nodes
                failed = []
                for node, status in nodes.iteritems():
                    if node != 'status' and status != 1:
                        failed.append(node)
                
                messages['error'].append("Volume %s is not healthy (failed nodes: %s)" % (vol, ', '.join(failed)))
                status = 'ERROR'
            else:
                messages['info'].append("Volume %s is healthy" % vol)

        self.result.add_component('Volumes', status, **messages)

    def getPeersStatus(self):
        """
        Get status of gluster peers

        Return dictionary of peers and their status:
        peers = {
            'gluster01': {
                'connected' : 1,
                'status' : 'Peer rejected',
            }
        }
        """
        peers = {}
        stdout, stderr, retval = self.execute('/usr/sbin/gluster peer status --xml', stderr=subprocess.STDOUT)

        try:
            xml = minidom.parseString(stdout)
        except Exception as e:
            # eg. No peers present
            raise Exception(stdout)

        for i in range(50):
            try:
                hostname = xml.getElementsByTagName('friend%d.hostname' % i)[0].toxml().replace('<friend%d.hostname>' % i, '').replace('</friend%d.hostname>' % i, '')
                connected   = xml.getElementsByTagName('friend%d.connected' % i)[0].toxml().replace('<friend%d.connected>' % i, '').replace('</friend%d.connected>' % i, '')
                status = xml.getElementsByTagName('friend%d.state' % i)[0].toxml().replace('<friend%d.state>' % i, '').replace('</friend%d.state>' % i, '')
            except:
                continue

            # Update connected status
            if int(connected) == 1:
                connected = True
            else:
                connected = False

            peers[hostname] = {
                'connected'  : connected,
                'status': status,
            }

        return peers

    def getVolumesStatus(self):
        """
        Get status for all volumes

        Return dictionary with status of every volume:
        status = {
            'Volume' : {
                'gluster01' : 1,
                'gluster01' : 1,
                'status'    : 1,
            }
        }
        """
        status  = {}
        volumes = self.getVolumes()
        for vol in volumes:
            status[vol] = self.getStatus(vol)

        return status

    def getVolumes(self):
        """
        Get list of available volumes
        """
        volumes = []
        stdout, stderr, retval = self.execute('/usr/sbin/gluster volume list', stderr=subprocess.STDOUT)
        for vol in stdout.split('\n'):
            vol = vol.replace('\n', '')
            if vol:
                volumes.append(vol)
        return volumes

    def getStatus(self, volume):
        """
        Get volume status

        Return dictionary with status of every node and overall status:
        status = {
            'gluster01' : 1,
            'gluster02' : 1,
            'status'    : 1,
        }
        """
        volStat = {
            'status' : 1,
        }

        ret = 1
        tries = 0
        retry = 5
        while ret != 0 and tries <= retry:
            stdout, stderr, retval = self.execute('/usr/sbin/gluster volume status %s --xml' % volume, stderr=subprocess.STDOUT)

            try:
                xml = minidom.parseString(stdout)
            except Exception as e:
                raise Exception("Unexpected output from gluster volume status %s --xml: %s" % (volume, stdout))

            # Can't get volume info, sleep for a while and try it again (only one command of this type can be run on cluster)
            ret = int(xml.getElementsByTagName('opRet')[0].toxml().replace('<opRet>', '').replace('</opRet>', ''))
            tries += 1
            time.sleep(random.uniform(0, 1))

        if ret != 0:
            raise Exception("Can't get data for %s after %d retries" % (volume, retry))

        nodes = xml.getElementsByTagName('node')

        for node in nodes:
            hostname = node.getElementsByTagName('hostname')[0].toxml().replace('<hostname>', '').replace('</hostname>', '')
            status   = int(node.getElementsByTagName('status')[0].toxml().replace('<status>', '').replace('</status>', ''))
            volStat[hostname] = status

            # Set failed status for volume
            if status != 1 and volStat['status'] == 1:
                volStat['status'] = 0

        return volStat

    def mount(self, volume):
        tmp = tempfile.mkdtemp()

        status = 'OK'
        messages = {
            'info' : [],
            'error': [],
            'warn' : [],
        }

        try:
            stdout, stderr, retval = self.execute('/bin/mount -t glusterfs localhost:%s %s' % (volume, tmp), stderr=subprocess.STDOUT)

            if retval:
                messages['error'].append(stderr)
                status = 'ERROR'
            else:
                # Unmount
                stdout, stderr, retval = self.execute('/bin/umount %s' % tmp, stderr=subprocess.STDOUT)

                if retval:
                    messages['error'].append(stderr)
                    status = 'ERROR'
            
            self.result.add_component('Mount', status, **messages)
        finally:
            os.rmdir(tmp)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2012, GoodData(R) Corporation. All rights reserved

import datetime
import logging
import multiprocessing
import os
import re
import signal
import simplejson
import setproctitle
import time
import types

from smoker.server.exceptions import *
import smoker.util.command

lg = logging.getLogger('smokerd.pluginmanager')


def alarm_handler(signum, frame):
    raise PluginExecutionTimeout


class PluginManager(object):
    """
    PluginManager provides management and
    access to plugins
    """
    # Configured plugins/templates/actions
    conf_plugins   = None
    conf_actions   = None
    conf_templates = None

    # Plugin objects
    plugins = {}

    # Processes
    processes = []
    # We don't want to have process ID 0 (first index)
    # so fill it by None
    processes.append(None)

    def __init__(self, plugins=None, actions=None, templates=None,
                 semaphore_count=None):
        """
        PluginManager constructor
         * load plugins/templates/actions configuration
         * create plugins objects
        """
        self.conf_plugins   = plugins
        self.conf_actions   = actions
        self.conf_templates = templates

        self.stopping = False

        # Initialize multiprocessing semamphore, by default limit by
        # number of online CPUs + 2
        if not semaphore_count:
            semaphore_count = int(os.sysconf('SC_NPROCESSORS_ONLN')) + 2
        lg.info("Plugins will run approximately at %s parallel processes",
                semaphore_count)
        global semaphore
        semaphore = multiprocessing.Semaphore(semaphore_count)

        # Load Plugin objects
        self.load_plugins()

    def stop(self, blocking=True):
        """
        Stop all plugins
        Wait until they are stopped if blocking=True
        """
        self.stopping = True

        # Trigger stop of all plugins
        for plugin in self.plugins.values():
            if plugin.current_run:
                plugin.current_run.stop()
                plugin.current_run.terminate()

        # Wait until all plugins are stopped
        if blocking:
            plugins_left = self.plugins.keys()
            plugins_left_cnt = len(plugins_left)
            while plugins_left:
                plugins_left = []
                for name, plugin in self.plugins.iteritems():
                    if not plugin.current_run:
                        continue
                    if plugin.current_run.is_alive():
                        plugins_left.append(name)
                    else:
                        plugin.current_run.join()
                if plugins_left:
                    # Print info only if number of left plugins changed
                    if len(plugins_left) != plugins_left_cnt:
                        lg.info("Waiting for %s plugins to shutdown: %s" % (len(plugins_left), ','.join(plugins_left)))
                    plugins_left_cnt = len(plugins_left)
                    time.sleep(0.5)

    def load_plugins(self):
        """
        Create Plugin objects
        """
        # Check if BasePlugin template is present
        # or raise exception
        try:
            self.get_template('BasePlugin')
        except (TemplateNotFound, NoTemplatesConfigured):
            lg.error("Required BasePlugin template is not configured!")
            raise BasePluginTemplateNotFound

        for plugin, options in self.conf_plugins.iteritems():
            if options.has_key('Enabled') and options['Enabled'] == False:
                lg.info("Plugin %s is disabled, skipping.." % plugin)
                continue

            try:
                self.plugins[plugin] = self.load_plugin(plugin, options)
            except TemplateNotFound:
                lg.error("Can't find configured template %s for plugin %s, plugin not loaded" % (options['Template'], plugin))
                continue
            except NoTemplatesConfigured:
                lg.error("There are no templates configured, template %s is required by plugin %s, plugin not loaded" % (options['Template'], plugin))
                continue
            except AssertionError as e:
                lg.error("Plugin %s not loaded: AssertionError, %s" % (plugin, e))
                continue
            except Exception as e:
                lg.error("Plugin %s not loaded: %s" % (plugin, e))
                lg.exception(e)
                continue
            lg.info("Loaded plugin %s" % plugin)

        if len(self.plugins) == 0:
            lg.error("No plugins loaded!")
            raise NoRunningPlugins("No plugins loaded!")

    def load_plugin(self, plugin, options):
        """
        Create and return Plugin object
        """
        # Load BasePlugin template first
        try:
            template = self.get_template('BasePlugin')
        except:
            template = {}

        # Plugin has template, load it's parent params
        if options.has_key('Template'):
            template_custom = self.get_template(options['Template'])
            template = dict(template, **template_custom)

        if options.has_key('Action'):
            options['Action'] = self.get_action(options['Action'])

        params = dict(template, **options)
        return Plugin(plugin, params)

    def get_template(self, name):
        """
        Return template parameters
        """
        if not isinstance(self.conf_templates, dict):
            raise NoTemplatesConfigured

        try:
            params = self.conf_templates[name]
        except KeyError:
            raise TemplateNotFound("Can't find configured template %s" % name)

        return params

    def get_action(self, name):
        """
        Return template parameters
        """
        if not isinstance(self.conf_actions, dict):
            raise NoActionsConfigured

        try:
            params = self.conf_actions[name]
        except KeyError:
            raise ActionNotFound("Can't find configured action %s" % name)

        return params

    def get_plugins(self, filter=None):
        """
        Return all plugins or filter them by parameter
        """

        if filter:
            plugins = []
            key = filter.keys()[0]
            value = filter[key]

            for plugin in self.plugins.itervalues():
                if plugin.params.has_key(key):
                    if plugin.params[key] == value:
                        plugins.append(plugin)
            return plugins
        else:
            return self.plugins

    def get_plugin(self, name):
        """
        Return single plugin
        """
        try:
            return self.plugins[name]
        except KeyError:
            raise NoSuchPlugin("Plugin %s not found" % name)

    def add_process(self, plugins=None, filter=None):
        """
        Add process and force plugin run
        """
        plugins_list = []

        # Add plugins by name
        if plugins:
            for plugin in plugins:
                plugins_list.append(self.get_plugin(plugin))

        # Add plugins by filter
        if filter:
            plugins_list.extend(self.get_plugins(filter))

        # Raise exception if no plugins was found
        if len(plugins_list) == 0:
            raise NoPluginsFound

        process = {
            'plugins' : plugins_list,
        }

        plugins_name = []
        for p in plugins_list:
            plugins_name.append(p.name)

        lg.info("Forcing run of %d plugins: %s" % (len(plugins_list), ', '.join(plugins_name)))

        # Add process into the list
        self.processes.append(process)
        id = len(self.processes)-1

        # Force run for each plugin and clear forced_result
        for plugin in plugins_list:
            plugin.forced = True
            plugin.forced_result = None
            plugin.run()

        return id

    def get_process(self, id):
        """
        Return process
        """
        return self.processes[id]

    def get_process_list(self):
        """
        Return all processes
        """
        return self.processes

    def run_plugins_with_interval(self):
        """
        Start run of plugins configured as such
        """
        for plugin in self.plugins.values():
            if not plugin.params['Interval']:
                continue
            plugin.run()

    def join_timed_plugin_workers(self):
        """
        Join zombie workers of interval-triggered runs
        The results will be picked by REST server forked ends of the queues
        """
        for plugin in self.plugins.values():
            if not plugin.params['Interval']:
                continue
            if plugin.current_run:
                plugin.current_run.join()
                plugin.current_run = None


class Plugin(object):
    """
    Object that represents single plugin
    """
    name = None
    params = {}
    current_run = None

    params_default = {
        'Command': None,
        'Module': None,
        'Parser': None,
        'Interval': 0,
        'Timeout': 1800,
        'History': 10,
        'uid': 'default',
        'gid': 'default',
        'Template': None,
        'Action': None,
    }

    def __init__(self, name, params):
        """
        Plugin constructor
         * prepare the process

        :param name: name of the plugin
        :type name: string

        :param params: keyword arguments
        :type params: dict
        """
        assert isinstance(name, basestring)
        assert isinstance(params, dict)
        self.name = name
        self.params = dict(self.params_default, **params)
        self.stopping = False

        # Set Action properly to have all
        # required default parameters
        if self.params['Action']:
            action_default = {
                'Command' : None,
                'Module'  : None,
                'Timeout' : 60,
            }
            self.params['Action'] = dict(action_default, **params['Action'])

        # create the instances of the Queue and force flag
        self.queue = multiprocessing.Queue()
        self.forced = False

        # Set those variables or they will be
        # references, shared between plugins
        self.result = []
        self.forced_result = None
        self.next_run = False

        # Validate configuration
        self.validate()

        # Schedule first plugin run
        if self.params['Interval']:
            self.schedule_run()

    def validate(self):
        """
        Validate configuration
        Raise InvalidConfiguration exception if invalid
        """
        # Timeout can't be 0 or less
        if self.params['Timeout'] <= 0:
            raise InvalidConfiguration("Timeout parameter can't be 0")

        # Command or Module have to be set
        if not self.params['Command'] and not self.params['Module']:
            raise InvalidConfiguration("Command or Module parameter has to be set")

        # Command and Module params can't be together
        if self.params['Command'] and self.params['Module']:
            raise InvalidConfiguration("Command and Module parameters cannot be set together")

        # Parser can't be set without command
        if not self.params['Command'] and self.params['Parser']:
            raise InvalidConfiguration("Parser can be used only with Command parameter")

    def run(self):
        """
        Run process
        Check if plugin should be run and execute it
        """
        if self.current_run:  # already running
            return
        # Plugin run when forced
        if self.forced:
            self.current_run = PluginWorker(self.name, self.queue, self.params,
                                            self.forced)
            self.current_run.start()
        elif self.params['Interval']:
            if datetime.datetime.now() >= self.next_run:
                self.current_run = PluginWorker(self.name, self.queue,
                                                self.params)
                self.current_run.start()
                self.schedule_run()

    def schedule_run(self, time=None, now=False):
        """
        Schedule next plugin run
        Accept datetime object as time parameter or set
        current time if now parameter is True
        """
        if time:
            if isinstance(time, object) and type(time).__name__ == 'datetime':
                self.next_run = time
            else:
                raise InvalidArgument(
                    'Parameter time has to be an instance of datetime object')
        elif now:
            self.next_run = datetime.datetime.now()
        elif self.params['Interval']:
            self.next_run = (
                datetime.datetime.now() +
                datetime.timedelta(seconds=self.params['Interval']))

    def collect_new_result(self):
        if self.queue.empty():  # no new results
            return

        while not self.queue.empty():
            result = self.queue.get()
            self.result.append(result)

            if 'forced' in result.keys() and result['forced']:
                self.forced_result = self.get_last_result()
                self.forced = False

            if len(self.result) > self.params['History']:
                self.result.pop(0)

        if self.current_run:  # forced, not externally fed to the queue
            self.current_run.join()
            self.current_run = None

    def get_last_result(self):
        """
        Get last run result or None
        """
        if not self.result:
            return None

        return self.result[-1]


class PluginWorker(multiprocessing.Process):
    def __init__(self, name, queue, params, forced=False):
        self.plugin_name = name
        self.queue = queue
        self.params = params
        self.forced = forced
        self.result = None

        super(PluginWorker, self).__init__()
        self.daemon = True

    def run(self):
        setproctitle.setproctitle('smokerd plugin %s' % self.plugin_name)
        self.drop_privileged()

        with semaphore:
            self.run_plugin(self.forced)
        self.queue.put(self.result)

    def run_command(self, command, timeout=0):
        """
        Run system command and parse output
        """
        result = Result()
        lg.debug("Plugin %s: executing command %s" % (self.name, command))

        try:
            stdout, stderr, returncode = smoker.util.command.execute(
                command, timeout=timeout)
        except smoker.util.command.ExecutionTimeout as e:
            raise PluginExecutionTimeout(e)
        except Exception as e:
            lg.exception(e)
            raise PluginExecutionError(
                "Can't execute command %s: %s" % (command, e))

        if returncode:
            status = 'ERROR'
        else:
            status = 'OK'

        # Run parser or parse output from stdin
        if self.params['Parser']:
            try:
                result = self.run_parser(stdout, stderr)
            except Exception as e:
                # Error result
                result.set_status('ERROR')
                result.add_error(re.sub('^\n', '', stderr.strip()))
                result.add_error('Parser run failed: %s' % e)
                result.add_info(re.sub('^\n', '', stdout.strip()))
        else:
            # Try to parse JSON output
            json = None
            try:
                json = simplejson.loads(stdout)
            except:
                pass

            if json:
                # Output is JSON, check it has valid status or raise exception
                if json.has_key('status') and json['status'] in [ 'OK', 'ERROR', 'WARN' ]:
                    try:
                        result.set_result(json, validate=True)
                    except ValidationError as e:
                        raise PluginMalformedOutput("Invalid JSON structure: %s" % e)
                else:
                    raise PluginMalformedOutput("Missing status in JSON output: %s" % json)
            else:
                # Output is not JSON, use stdout/stderr and return value
                lg.debug("Plugin %s: using non-JSON output" % self.name)
                result.set_status(status)
                if stderr:
                    result.add_error(re.sub('^\n', '', stderr.strip()))
                if stdout:
                    result.add_info(re.sub('^\n', '', stdout.strip()))

        return result

    def run_parser(self, stdout, stderr):
        """
        Run parser on given stdout/stderr
        Raise exceptions if anything happen
        """
        lg.debug("Plugin %s: running parser %s" % (self.name, self.params['Parser']))

        if stdout:
            lg.debug("Plugin %s: stdout: %s" % (self.name, stdout.strip()))
        if stderr:
            lg.debug("Plugin %s: stderr: %s" % (self.name, stderr.strip()))

        try:
            parser = __import__(self.params['Parser'], globals(), locals(), ['Parser'], -1)
        except ImportError as e:
            lg.error("Plugin %s: can't load parser %s: %s" % (self.name, self.params['Parser'], e))
            raise

        try:
            parser = parser.Parser(stdout, stderr)
        except Exception as e:
            lg.error("Plugin %s: can't initialize parser: %s" % (self.name, e))
            lg.exception(e)
            raise

        try:
            result = parser.parse()
        except Exception as e:
            lg.error("Plugin %s: parser execution failed: %s" % (self.name, e))
            lg.exception(e)
            raise

        return result

    def run_module(self, module, **kwargs):
        """
        Run Python module
        Raise exceptions if anything happen
        """
        lg.debug("Plugin %s: running module %s" % (self.name, module))
        try:
            plugin = __import__(module, globals(), locals(), ['Plugin'], -1)
        except ImportError as e:
            lg.error("Plugin %s: can't load module %s: %s" % (self.name, module, e))
            raise

        try:
            plugin = plugin.Plugin(self, **kwargs)
        except Exception as e:
            lg.error("Plugin %s: can't initialize plugin module: %s" % (self.name, e))
            lg.exception(e)
            raise

        signal.signal(signal.SIGALRM, alarm_handler)
        if 'timeout' not in kwargs:
            kwargs['timeout'] = self.get_param('Timeout', default=120)

        try:
            signal.alarm(kwargs['timeout'])
            result = plugin.run()
        except PluginExecutionTimeout:
            result = Result()
            result.set_status('ERROR')
            result.add_error('Plugin execution exceeded timeout %d seconds' %
                             kwargs['timeout'])
        except Exception as e:
            lg.error("Plugin %s: module execution failed: %s" % (self.name, e))
            lg.exception(e)
            signal.alarm(0)
            raise

        signal.alarm(0)
        return result

    def run_plugin(self, force=False):
        """
        Run plugin, save result and schedule next run

        :param force: forced run
        :type force: bool
        """
        # External command will be executed
        if self.params['Command']:
            command = self.params['Command'] % self.escape(dict(self.params))
            # Execute external command to get result
            try:
                result = self.run_command(command, self.params['Timeout'])
            except Exception as e:
                lg.error("Plugin %s: %s" % (self.name, e))
                result = Result()
                result.set_status('ERROR')
                result.add_error(e)
        # Python module will be executed
        elif self.params['Module']:
            try:
                result = self.run_module(self.params['Module'])
            except Exception as e:
                lg.error("Plugin %s: %s" % (self.name, e))
                result = Result()
                result.set_status('ERROR')
                result.add_error(re.sub('^\n', '', ('%s' % e).strip()))
        # No module or command to run
        else:
            lg.error("Plugin %s: no Command or Module to execute!" % self.name)
            result = Result()
            result.set_status('ERROR')
            result.add_error('No Command or Module to execute!')

        # Run action on result
        if self.params['Action']:
            lg.debug("Plugin %s: executing action" % self.name)
            # Execute external command
            if self.params['Action']['Command']:
                # Add parameters to command with format
                params = dict(self.params, **result)
                params = self.escape(params)

                try:
                    action = self.run_command(self.params['Action']['Command'] % params, timeout=self.params['Action']['Timeout'])
                except Exception as e:
                    lg.error("Plugin %s: %s" % (self.name, e))
                    action = Result()
                    action.set_status('ERROR')
                    action.add_error(e)
            # Execute Python module
            elif self.params['Action']['Module']:
                try:
                    action = self.run_module(self.params['Action']['Module'], result=result)
                except Exception as e:
                    lg.error("Plugin %s: %s" % (self.name, e))
                    action = Result()
                    action.set_status('ERROR')
                    action.add_error(e)
            # No command or module to execute
            else:
                lg.error("Plugin %s: no Action Command or Module to execute!" % self.name)
                action = Result()
                action.set_status('ERROR')
                action.add_error('No Command or Module to execute!')
            # Add action result to plugin result
            result.set_action(action)

        result.set_forced(force)
        # send to the daemon
        try:
            self.result = result.get_result()
        except ValidationError as e:
            lg.error("Plugin %s: ValidationError: %s" % (self.name, e))
            result = Result()
            result.set_status('ERROR')
            result.add_error('ValidationError: %s' % e)
            result.set_forced(force)
            self.result = result.get_result()

        # Log result
        lg.info("Plugin %s result: %s" % (self.name, result.get_result()))

    def escape(self, tbe):
        """
        Escape given string, dictionary or list
        If int, None or bool item is found, just pass
        Also pass if item can't be escaped by some other reason
        Raise exception if unknown data type
        """
        if isinstance(tbe, dict):
            escaped = {}
            for key, value in tbe.iteritems():
                if type(value) in [int, types.NoneType, bool]:
                    escaped[key] = value
                else:
                    try:
                        escaped[key] = re.escape(value)
                    except:
                        escaped[key] = value
        elif isinstance(tbe, basestring):
            try:
                escaped = re.escape(tbe)
            except:
                escaped = tbe
        elif isinstance(tbe, int) or isinstance(tbe, bool):
            escaped = tbe
        elif isinstance(tbe, list):
            escaped = []
            for value in tbe:
                if type(value) in [int, types.NoneType, bool]:
                    escaped.append(value)
                else:
                    try:
                        escaped.append(re.escape(value))
                    except:
                        escaped.append(value)
        else:
            raise Exception("Unknown data type")

        return escaped

    def get_param(self, name, default=None):
        """
        Get plugin parameter
        Return default if parameter doesn't exist
        """
        try:
            return self.params[name]
        except KeyError:
            return default

    def drop_privileged(self):
        if (self.params['uid'] == 'default' and
                self.params['gid'] == 'default'):
            return
        lg.debug("Plugin %s: dropping privileges to %s/%s"
                 % (self.name, self.params['uid'], self.params['gid']))
        try:
            os.setegid(self.params['gid'])
            os.seteuid(self.params['uid'])
        except TypeError as e:
            lg.error("Plugin %s: config parameters uid/gid have to be "
                     "integers: %s" % (self.name, e))
            raise
        except OSError as e:
            lg.error("Plugin %s: can't switch effective UID/GID to %s/%s: %s"
                     % (self.name, self.params['uid'], self.params['gid'], e))
            raise


class Result(object):
    """
    Object that represents plugin result
    """
    validated = False

    def __init__(self):
        """
        Default result values
        """
        self.result = {
            'status': None,
            'messages': None,
            'lastRun': datetime.datetime.now().isoformat(),
            'componentResults': None,
            'action': None,
            'forced': False
        }

    def set_status(self, status=None):
        """
        Set result status
        If status is empty, generate it from componentResults
        """
        if status is None:
            if not self.result['componentResults']:
                raise Exception("Can't generate overall status without component results")
            status = self._gen_component_status()

        if status not in ['OK', 'ERROR', 'WARN']:
            raise InvalidArgument("Status has to be OK, ERROR or WARN")

        self.result['status'] = status

    def _gen_component_status(self, default='OK'):
        """
        Generate status from component results
        """
        status = default
        for result in self.result['componentResults'].itervalues():
            if result['status'] == 'OK' and status not in ['WARN', 'ERROR']:
                status = 'OK'
            elif result['status'] == 'WARN' and status != 'ERROR':
                status = 'WARN'
            elif result['status'] == 'ERROR':
                status = 'ERROR'
        return status

    def set_forced(self, forced=True):
        self.result['forced'] = forced

    def add_info(self, msg):
        """
        Add info messge
        """
        self.add_msg('info', msg)

    def add_error(self, msg):
        """
        Add error messge
        """
        self.add_msg('error', msg)

    def add_warn(self, msg):
        """
        Add warn messge
        """
        self.add_msg('warn', msg)

    def add_msg(self, level, msg, multiline=False):
        """
        Add message into result
        Empty messages are skipped

        multiline - don't split message lines into
            multiple messages
        """
        # Create messages structure if it doesn't exists
        if not self.result['messages']:
            self.result['messages'] = {
                'info' : [],
                'error': [],
                'warn' : [],
            }

        if not multiline:
            messages = str(msg).split('\n')
        else:
            messages = [str(msg)]

        for message in messages:
            # Skip adding empty message
            if not str(msg).strip():
                continue

            try:
                self.result['messages'][level].append(str(message).strip())
            except KeyError:
                raise InvalidArgument("Level has to be info, error or warn")

    def validate(self, force=False):
        """
        Validate result
        Skip if it was already validated to avoid
        unwanted re-validation
        """
        if force != True and self.validated == True:
            return True
        else:
            try:
                self._validate_status(self.result['status'])
                self._validate_msg(self.result['messages'])
                self._validate_component_result(self.result['componentResults'])
                self._validate_action(self.result['action'])
            finally:
                self.validated = True

    def _validate_status(self, status):
        """
        Validate result status
        """
        if status not in ['OK', 'ERROR', 'WARN']:
            raise ValidationError("Result status has to be OK, ERROR or WARN, not %s" % status)

    def _validate_msg(self, msg):
        types = ['info', 'error', 'warn']
        if type(msg).__name__ not in ['NoneType', 'dict']:
            raise ValidationError("Result message has to be a dictionary or None, not %s" % type(msg).__name__)

        if msg:
            # Check every output type and validate it
            for t in types:
                try:
                    if not isinstance(msg[t], list):
                        raise ValidationError("Result message type %s has to be a list, not %s" % (t, type(msg[t]).__name__))
                    for out in msg[t]:
                        if not isinstance(out, basestring):
                            raise ValidationError("Result message type %s has to be a string, not %s" % (t, type(out).__name__))
                except Exception as e:
                    raise ValidationError("Can't validate message: %s" % e)

    def _validate_component_result(self, result):
        """
        Validate componentResults
        """
        # Component result can be empty
        if result == None:
            return True

        if not isinstance(result, dict):
            raise ValidationError("Component result must be dictionary")

        for name, component in result.iteritems():
            try:
                self._validate_msg(component['messages'])
            except KeyError:
                raise ValidationError("Component %s doesn't have message" % name)

            try:
                self._validate_status(component['status'])
            except KeyError:
                raise ValidationError("Component %s doesn't have status" % name)

    def _validate_action(self, result):
        """
        Validate action result
        """
        # Action can be empty
        if result == None:
            return True

        if not isinstance(result, dict):
            raise ValidationError("Action result must be dictionary")

        try:
            self._validate_msg(result['messages'])
        except KeyError:
            raise ValidationError("Action doesn't have message")

        try:
            self._validate_status(result['status'])
        except KeyError:
            raise ValidationError("Action doesn't have status")

    def set_result(self, result, validate=False):
        """
        Set result
        Not all fields have to be filled

        This method should be rarely used because it can
        cause invalid results

        Use validate=True and catch ValidationError to ensure
        given result is correct

        Result = <{
            'status'   : 'OK' | 'WARN' | 'ERROR',
            'messages' : Messages | NULL,
            'componentResults' : (STRING : {
                'status'  : 'OK' | 'WARN' | 'ERROR',
                'messages': Messages
            })* | NULL,
            'action'   : Result | NULL
        }>

        Messages = {
            'info' : [STRING],
            'error': [STRING],
            'warn' : [STRING]
        }
        """
        fields = [ 'status', 'messages', 'componentResults', 'action' ]

        for field in fields:
            try:
                self.result[field] = result[field]
            except KeyError:
                # just skip missing fields
                pass

        if validate == True:
            self.validate()

    def get_result(self, validate=True):
        """
        Validate by default and return result
        """
        if validate == True:
            self.validate()
        return self.result

    def set_action(self, action):
        """
        Set action result
        """
        if isinstance(action, object):
            self.result['action'] = action.get_result()
        elif isinstance(action, dict):
            self.result['action'] = action
        else:
            raise InvalidArgument("Action has to be a dict or a Result object")

    def add_component(self, name, status, info=[], error=[], warn=[]):
        """
        Add component result
        """
        if not self.result['componentResults']:
            self.result['componentResults'] = {}

        self.result['componentResults'][name] = {
            'status': status,
            'messages'   : {
                'info' : info,
                'error': error,
                'warn' : warn,
            },
        }

class BasePlugin(object):
    """
    Base class for all Python plugins
    """
    plugin = None
    result = None
    args   = None

    def __init__(self, plugin, **kwargs):
        """
        Set plugin parrent
        """
        self.plugin = plugin
        self.result = Result()

        self.args = kwargs

    def get_result(self):
        """
        Get and validate result
        Raise exception if it's not valid
        """
        return self.result.get_result()

    def execute(self, command, **kwargs):
        """
        Execute smoker.util.command.execute() with timeout (default 120 seconds)
        You shouldn't use anything else than this function from inside plugins!
        """
        # Set default timeout
        if not kwargs.has_key('timeout'):
            kwargs['timeout'] = self.plugin.get_param('Timeout', default=120)

        return smoker.util.command.execute(command, **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2012, GoodData(R) Corporation. All rights reserved

from smoker.server.plugins import Result

class BaseParser(object):
    """
    Base class for all parsers
    """
    stdout = None
    stderr = None
    result = None

    def __init__(self, stdout, stderr):
        """
        Set stdout and stderr
        """
        self.stdout = stdout
        self.stderr = stderr
        self.result = Result()

    def get_result(self):
        """
        Get result
        """
        return self.result.get_result()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2012, GoodData(R) Corporation. All rights reserved

"""
Connect to Mongo, test it's status

Parameters:
    Host    - host to connect to (default localhost)
    Port    - port to connect to (required)
    ValidateCollections - validate collections or not (default False)
    EmptyCollections    - if true, warn on empty collections (default False)
    ReplicaSet  - replica set to use (default text)
"""

import logging

from smoker.server.plugins import BasePlugin
lg = logging.getLogger('smokerd.plugin.mongo')

import datetime
import pymongo

class Plugin(BasePlugin):
    connection = None
    validate   = False
    host  = None
    port  = None
    empty = False
    replicaset = None

    def run(self):
        # Get parameters
        self.host = self.plugin.get_param('Host', 'localhost')
        self.port = self.plugin.get_param('Port', 27017)
        self.validate_collections = self.plugin.get_param('ValidateCollections')
        self.empty_collections = self.plugin.get_param('EmptyCollections')
        self.replicaset = self.plugin.get_param('ReplicaSet', 'test')

        # Define component checks
        checks = [
            { 'Connection' : self.check_connection },
            { 'Members' : self.check_members },
            { 'Collections' : self.check_collections },
            { 'Record' : self.check_record }
        ]

        # Check components
        for item in checks:
            for check, check_function in item.iteritems():
                try:
                    res = check_function()
                    self.result.add_component(check, res['state'], **res['messages'])
                except Exception as e:
                    self.result.add_component(check, 'ERROR', error=[str(e)])
                    lg.exception(e)

        self.result.set_status()
        return self.result

    def check_record(self):
        """
        Try to write and read from collection
        """
        result = {
            'state' : 'OK',
            'messages': {
                'info'  : [],
                'error' : [],
                'warn'  : [],
            }
        }

        # Use database test
        db = self.connection['test']

        # Use collection test
        collection = db.test

        # Write new post
        try:
            id = collection.insert({
                'name' : 'John Doe',
                'text' : 'Testing record',
                'date' : datetime.datetime.now()
            })
            result['messages']['info'].append("Inserted record with id %s" % id)
        except Exception as e:
            result['messages']['error'].append("Inserting record into collection failed: %s" % e)
            result['state'] = 'ERROR'
            return result

        # Read our post
        post = collection.find_one()
        if post:
            result['messages']['info'].append("Found our record: %s" % post)
        else:
            result['messages']['error'].append("Inserted record not found!")
            result['state'] = 'ERROR'

        # Empty collection
        try:
            collection.remove()
            result['messages']['info'].append("Deleted everything from collection test")
        except Exception as e:
            result['messages']['error'].append("Can't delete everything from collection test: %s" % e)
            result['state'] = 'ERROR'

        return result

    def check_collections(self):
        """
        Check collections in all databases
        """
        result = {
            'state' : 'OK',
            'messages' : {
                'info' : [],
                'error': [],
                'warn' : [],
            }
        }

        databases = self.connection.database_names()

        if not databases:
            result['state'] = 'WARN'
            result['messages']['warn'].append('No databases found')

        # Check each database and it's collections
        for database in databases:
            db = self.connection[database]
            collections = db.collection_names()

            if self.empty_collections and not collections:
                # No collections in database - WARN and continue
                if database == 'local':
                    # ignore on database local - it's OK to be empty
                    if result['state'] not in ['ERROR', 'WARN']:
                        result['state'] = 'OK'
                    result['messages']['info'].append('Database %s: no collections found but it is expected' % database)
                    continue
                else:
                    if result['state'] != 'ERROR':
                        result['state'] = 'WARN'
                    result['messages']['warn'].append('Database %s: no collections found, empty database' % database)
                    continue

            # Validate collections
            if self.validate_collections:
                for collection in collections:
                    validation = db.validate_collection(collection)
                    if validation['valid']:
                        result['messages']['info'].append('Database %s: collection %s is valid' % (database, collection))
                        if result['state'] not in ['ERROR', 'WARN']:
                            result['state'] = 'OK'
                    else:
                        for e in validation['errors']:
                            result['messages']['error'].append('Database %s: %s' % (database, e))
                        result['state'] = 'ERROR'
            else:
                if result['state'] not in ['ERROR', 'WARN']:
                    result['state'] = 'OK'
                result['messages']['info'].append('Database %s: %d collections found' % (database, len(collections)))

        return result

    def check_connection(self):
        """
        Connect to Mongo, save connection in self.connection
        and return status and message
        """
        try:
            # Create connection and allow reading data from slave
            self.connection = pymongo.Connection(self.host, self.port, slave_okay=True, replicaSet=self.replicaset)
        except Exception as e:
            return {
                'state': 'ERROR',
                'messages' : {
                    'error' : [str(e)]
                }
            }

        return {
            'state': 'OK',
            'messages' : {
                'info': [self.connection.server_info()['sysInfo']]
            }
        }

    def check_members(self):
        """
        Check MongoDB peers and their status
        """
        if not self.connection:
            raise Exception('Not connected')

        db_admin = self.connection.admin

        try:
            status = db_admin.command('replSetGetStatus')
        except pymongo.errors.OperationFailure as e:
            # If Mongo isn't running clustered, return WARN, otherwise raise exception
            if str(e) == '''command SON([('replSetGetStatus', 1)]) failed: not running with --replSet''':
                return {
                    'state' : 'OK',
                    'messages' : {
                        'info' : ['Mongo is not running clustered']
                    }
                }
            else:
                raise

        # Check status of Mongo peers
        members = {
            'state' : None,
            'messages' : {
                'info' : [],
                'error': [],
                'warn' : [],
            }
        }

        for member in status['members']:
            if member.has_key('errmsg'):
                msg = 'Member %s is %s: %s' % (member['name'], member['stateStr'], member['errmsg'])
            else:
                msg = 'Member %s is %s' % (member['name'], member['stateStr'])

            # Check member state
            #
            #   0  WARN   Starting up, phase 1 (parsing configuration)
            #   1  OK     Primary
            #   2  OK     Secondary
            #   3  WARN   Recovering (initial syncing, post-rollback, stale members)
            #   4  ERROR  Fatal error
            #   5  WARN   Starting up, phase 2 (forking threads)
            #   6  ERROR  Unknown state (the set has never connected to the member)
            #   7  OK     Arbiter
            #   8  ERROR  Down
            #   9  WARN   Rollback
            #  10  ERROR  Removed
            if member['state'] in [1, 2, 7]:
                # Member is PRIMARY or SECONDARY
                members['messages']['info'].append(msg)

                if members['state'] not in ['ERROR', 'WARN']:
                    members['state'] = 'OK'
            elif member['state'] in [0, 3, 5, 9]:
                # Warning only
                members['messages']['warn'].append(msg)

                if members['state'] != 'ERROR':
                    members['state'] = 'WARN'
            else:
                # Error
                members['messages']['error'].append(msg)
                members['state'] = 'ERROR'

        return members



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2013, GoodData(R) Corporation. All rights reserved

import sys

EXIT_OK = 0
EXIT_WARNING  = 1
EXIT_CRITICAL = 2
EXIT_UNKNOWN  = 3

def exit_critical(msg):
    print "CRITICAL: %s" % msg
    sys.exit(EXIT_CRITICAL)

def exit_warning(msg):
    print "WARNING: %s" % msg
    sys.exit(EXIT_WARNING)

def exit_unknown(msg):
    print "UNKNOWN: %s" % msg
    sys.exit(EXIT_UNKNOWN)

def exit_ok(msg):
    print "OK: %s" % msg
    sys.exit(EXIT_OK)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2012, GoodData(R) Corporation. All rights reserved

import argparse
import logging
import sys

from smoker.server.daemon import Smokerd
import smoker.logger


def main():
    parser = argparse.ArgumentParser(
        description='Smoke testing daemon',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument(
        '-c', '--config', dest='config', default='/etc/smokerd/smokerd.yaml',
        help="Config file to be used")
    parser.add_argument(
        '-p', '--pidfile', dest='pidfile', default='/var/run/smokerd.pid',
        help="PID file to be used")
    parser.add_argument(
        '--logging-conf', dest='logging_conf',
        default='/etc/smokerd/logging.ini', help="Path to logging config file")
    parser.add_argument('-fg', '--foreground', dest='foreground',
                        action='store_true', help="Don't fork into background")
    parser.add_argument('--stop', dest='stop', action='store_true',
                        help="Stop currently running daemon")
    parser.add_argument('--no-syslog', dest='no_syslog', action='store_true',
                        help=("Don't use syslog handler. Not effective when "
                              "logging config file is used."))
    parser.add_argument('-v', '--verbose', dest='verbose', action='store_true',
                        help="Be verbose")
    parser.add_argument('-d', '--debug', dest='debug', action='store_true',
                        help="Debug output")
    args = parser.parse_args()

    logging_args = {'config_file': args.logging_conf}
    # Don't log into console if we are going to background
    if not args.foreground:
        logging_args['console'] = False
    if args.no_syslog:
        logging_args['syslog'] = False

    lg = smoker.logger.init(**logging_args)

    if args.verbose:
        lg.setLevel(logging.INFO)

    if args.debug:
        lg.setLevel(logging.DEBUG)

    daemon = Smokerd(config=args.config, pidfile=args.pidfile)

    if args.stop:
        daemon.stop()
        sys.exit(0)

    if args.foreground:
        daemon.run()
    else:
        daemon.daemonize()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2013, GoodData(R) Corporation. All rights reserved

"""
Module tap provides interface for generating TAP (Test Anything Protocol) outputs

http://en.wikipedia.org/wiki/Test_Anything_Protocol


Usage example:

from util.tap import TapTest, Tap

# Host
tap_host1 = TapTest('host1', ok=True)

# Host's tests
tap_host1_tests = {
    'test1' : TapTest('test1', ok=True),
    'test2' : TapTest('test1', ok=True)
}
tap_host1.add_subtests(tap_host1_tests.values())

# Subtests for test2
tap_host_test2_subtest1 = TapTest('subtest1', ok=True)
tap_host1_tests['test2'].add_subtest(tap_host_test2_subtest1)

# Initialize Tap instance and add it into group
tap = Tap()
tap.add_test(tap_host1)

# Dump result
print tap.dump()

# Output will be:
# 1..1
# ok 1 - host1
#   1..2
#   ok 1 - test1
#   ok 2 - test1
#       1..1
#       ok 1 - subtest1
"""

import re

class Tap(object):
    """
    Class to group TapTest objects
    """
    tests = None
    index = None

    def __init__(self):
        """
        Initialize Tap instance
        """
        self.index = 1
        self.tests = []

    def add_test(self, test):
        """
        Add TapTest instance
        Return test index
        """
        assert isinstance(test, TapTest), 'test parameter must be instance of TapTest'

        test.index = self.index
        self.tests.append(test)
        self.index += 1

        return test.index

    def add_tests(self, tests):
        """
        Add list of TapTest instances
        """
        assert isinstance(tests, list), 'tests parameter must be list'

        for test in tests:
            assert isinstance(test, TapTest), 'test %s is not instance of TapTest' % test
            self.add_test(test)

    def dump(self):
        """
        Dump all tests into tap structure
        """
        dump = []

        dump.append('1..%s' % len(self.tests))
        for test in self.tests:
            dump.append(test.dump())

        return '\n'.join(dump)

class TapTest(object):
    """
    Single TAP test result
    """
    name = None
    status = None
    messages = None
    index = None

    subtests = None
    subtests_index = None

    def __init__(self, name, ok=True, messages=None):
        """
        Initialize instance
        """
        self.name = name
        self.index = 1
        self.subtests_index = 1
        self.subtests = []

        if ok is True:
            self.status = 'ok'
        else:
            self.status = 'not ok'
        self.messages = messages

    def add_subtest(self, test):
        """
        Add TapTest instance as subtest
        Return subtest index
        """
        test.index = self.subtests_index
        self.subtests.append(test)
        self.subtests_index += 1

        return test.index

    def add_subtests(self, tests):
        """
        Add list of TapTest instances
        """
        assert isinstance(tests, list), 'tests parameter must be list'

        for test in tests:
            assert isinstance(test, TapTest), 'test %s is not instance of TapTest' % test
            self.add_subtest(test)

    def dump(self):
        """
        Dump TapTest result into TAP
        Should be used from Tap instance to dump all results

        Dump messages in YAMLish format
        """
        dump = []
        dump.append('%s %s - %s' % (self.status, self.index, self.name))
        if self.messages:
            messages = []
            for key, values in self.messages.iteritems():
                if values:
                    messages.append('\t- %s:' % key)
                    for msg in values:
                        messages.append("\t\t- %s" % msg)
            if messages:
                messages.insert(0, '---')
                messages.append('...')
                dump.append('\n'.join(messages))
        if self.subtests:
            dump_subtests = []
            dump_subtests.append('\t1..%s' % len(self.subtests))
            for test in self.subtests:
                dump_subtests.append(test.dump())
            dump.append(re.sub('\n', '\n\t', '\n'.join(dump_subtests)))

        return '\n'.join(dump)



# -*- coding: utf-8 -*-
# Copyright (C) 2007-2014, GoodData(R) Corporation. All rights reserved

"""
Try to connect to given set of host, port and report status.

Parameters:
    Connections - Dictionary of [host, port] dict to try to connect to, example
                  [ ['icinga.mydomain',5556], ['myhost.mydomain',80 ] ]
    Timeout - timeout for requests, default 10 seconds

"""

from smoker.server.plugins import BasePlugin

import logging
lg = logging.getLogger('smokerd.plugin.testconnection')

import threading
import socket


class Plugin(BasePlugin):
    url = None
    params = None
    suite = None
    timeout = None

    def run(self):
        """
        Run plugin
         * get parameters and set defaults
         * run ConnectionCheck for all host, port combinations in parallel
        """
        # Get parameters
        self.addresses = self.plugin.get_param('Connections', [])
        self.timeout = self.plugin.get_param('Timeout', 10)

        # Defaults
        if not self.addresses and not isinstance(self.addresses, dict):
            raise Exception('Parameter Connections have to be dictionary of '
                            '(host, port) tuples')

        for conn in self.addresses:
            if (not (isinstance(conn, tuple) or isinstance(conn, list)) and
                    len(conn) != 2):
                raise Exception(
                    "Every parameter has to be a [host, port] list")

        threads = []
        for conn in self.addresses:
            t = ConnectionCheck((conn[0], conn[1]), self.timeout)
            t.start()
            threads.append(t)
        for t in threads:
            t.join()
        err = []
        for t in threads:
            err.extend(t.err)
        status = "OK"
        if err:
            status = "ERROR"
        self.result.add_component('Connection', status, error=err)
        self.result.set_status()
        return self.result


class ConnectionCheck(threading.Thread):
    """
    Opens socket to supplied address and check the return status,
    leaving results in self.err and self.info array.
    """
    def __init__(self, address, timeout):
        threading.Thread.__init__(self)
        self.timeout = timeout
        self.address = address
        self.err = []
        self.info = []

    def run(self):
        try:
            s = socket.create_connection(self.address, self.timeout)
            s.close()
            self.info.append("Host: %s" % self.address[0])
        except (socket.herror, socket.gaierror) as e:
            self.err.append(
                "Host: %s resolving error: %s" % (self.address[0], e))
        except socket.timeout as e:
            self.err.append(
                "Host: %s, port: %s connection timeout" % self.address)
        except socket.error as e:
            self.err.append("Host: %s, port: %s, Socket error: %s" %
                            (self.address[0], self.address[1], e))
        except Exception as e:
            lg.exception()
            self.err.append(
                "Host: %s, Unknown exception: %s" % (self.address[0], e))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2013, GoodData(R) Corporation. All rights reserved

"""
Check Smoker plugin and force it's run by option.
Return Nagios-friendly result. We can't send much data so just send if it's alright or not or print first line of
standard non-component messages if present.
"""

import argparse
import logging

from smoker.client import Client
import smoker.util.nagios as nagios
import smoker.logger


# Don't log into console, use only syslog
smoker.logger.init(syslog=True, console=False)
lg = logging.getLogger('check_smoker_plugin')

parser = argparse.ArgumentParser(description='Smoker Nagios plugin')
parser.add_argument('-f', '--force', dest='force', action='store_true', help="Force plugins run (otherwise just print last results)")
parser.add_argument('plugin', help="Plugin to check")
args = parser.parse_args()

def ParseIssue(plugin_result):
    out = "smoke test: %s " % plugin_result['name']
    for component in plugin_result['lastResult']['componentResults']:
        if component['componentResult']['status'] != 'OK':
            out += "%s: %s; " % ( component['componentResult']['name'], component['componentResult']['status'] )
    if len(out) > 1024:
        out = "smoke test: %s returns huge error output, see smoker logs for details" % plugin_result['name']
    return out

def main():
    client = Client(['localhost'])
    plugins = client.get_plugins([[args.plugin]])

    # No plugin found
    if not plugins:
        nagios.exit_unknown("plugin %s not found" % args.plugin)

    if args.force:
        plugins = client.force_run(plugins, progress=False)

    # Manage plugin result. We can't return much data to Nagios, so just say if it's alright or not
    for plugin in plugins.values()[0]['plugins'].itervalues():
        if not plugin['lastResult']:
            # No result?! -> UNKNOWN
            nagios.exit_unknown("plugin has no last result")
        elif plugin['lastResult']['status'] == 'OK':
            try:
                nagios.exit_ok(plugin['lastResult']['messages']['ok'][0])
            except (KeyError, IndexError, TypeError):
                nagios.exit_ok("smoke test %s succeeded at %s" % ( plugin['name'], plugin['lastResult']['lastRun']) )
        elif plugin['lastResult']['status'] == 'WARN':
            try:
                nagios.exit_warning(plugin['lastResult']['messages']['warn'][0])
            except (KeyError, IndexError, TypeError):
                nagios.exit_warning(ParseIssue(plugin))
        elif plugin['lastResult']['status'] == 'ERROR':
            try:
                nagios.exit_critical(plugin['lastResult']['messages']['error'][0])
            except (KeyError, IndexError, TypeError):
                nagios.exit_critical(ParseIssue(plugin))
        else:
            nagios.exit_unknown("unknown status %s at %s" % (plugin['lastResult']['status'], plugin['lastResult']['lastRun']))

if __name__ == '__main__':
    try:
        main()
    except Exception as e:
        lg.exception(e)
        nagios.exit_unknown(e)
